import urllib.parse
import urllib.request
import urllib.error
import random
import json
import sys
import grpc
import struct
import fnmatch
import os
import os.path
import ssl
import socket
from google.protobuf import text_format
from argparse import FileType
from functools import wraps
from inspect import signature
from operator import attrgetter, itemgetter
from collections import defaultdict
import ydb.core.protos.grpc_pb2_grpc as kikimr_grpc
import ydb.core.protos.msgbus_pb2 as kikimr_msgbus
import ydb.core.protos.blobstorage_config_pb2 as kikimr_bsconfig
import ydb.core.protos.cms_pb2 as kikimr_cms
import typing


bad_hosts = set()
cache = {}
name_cache = {}


class EndpointInfo:
    def __init__(self, protocol: str, host: str, port: int):
        self.protocol = protocol
        self.port = port
        self.host = host


class ConnectionParams:
    ENDPOINT_HELP = 'Default protocol is http, default port is 8765'

    def __init__(self):
        self.hosts = set()
        self.endpoints = dict()
        self.grpc_port = None
        self.mon_port = None
        self.mon_protocol = None
        self.token = None
        self.domain = None
        self.verbose = None
        self.quiet = None
        self.http_timeout = None
        self.cafile = None
        self.insecure = None
        self.http = None

    def make_endpoint_info(self, endpoint: str):
        return EndpointInfo(*self.get_protocol_host_port(endpoint))

    def get_protocol_host_port(self, endpoint):
        protocol, sep, endpoint = endpoint.rpartition('://')
        if sep != '://':
            protocol = self.mon_protocol if self.mon_protocol is not None else 'http'
        endpoint, sep, port = endpoint.partition(':')
        if sep == ':':
            return protocol, endpoint, int(port)
        else:
            return protocol, endpoint, self.mon_port

    def get_netloc(self, host, port):
        netloc = '%s:%d' % (host, port)
        if netloc in name_cache:
            netloc = name_cache[netloc]
        else:
            for af, socktype, proto, canonname, sa in socket.getaddrinfo(host, port, socket.AF_UNSPEC, socket.SOCK_STREAM, 0, socket.AI_PASSIVE):
                host, port = socket.getnameinfo(sa, socket.NI_NUMERICHOST | socket.NI_NUMERICSERV)
                if af == socket.AF_INET6:
                    host = '[%s]' % host
                new_netloc = '%s:%s' % (host, port)
                name_cache[netloc] = new_netloc
                netloc = new_netloc
        return netloc

    def make_url(self, host, path, params):
        endpoint_info = self.endpoints[host] if host in self.endpoints else self.make_endpoint_info(host)
        netloc = self.get_netloc(endpoint_info.host, endpoint_info.port)
        return urllib.parse.urlunsplit((endpoint_info.protocol, netloc, path, urllib.parse.urlencode(params), ''))

    def apply_args(self, args, with_localhost=True):
        self.grpc_port = args.grpc_port
        self.mon_port = args.mon_port
        self.mon_protocol = args.mon_protocol

        if args.endpoint:
            for endpoint in args.endpoint:
                endpoint_info = self.make_endpoint_info(endpoint)
                if self.mon_protocol is None:
                    self.mon_protocol = endpoint_info.protocol
                host_with_port = '{0}:{1}'.format(endpoint_info.host, endpoint_info.port)
                self.hosts.add(endpoint_info.host)
                self.endpoints[endpoint_info.host] = endpoint_info
                self.endpoints[host_with_port] = endpoint_info

        if self.mon_protocol is None:
            self.mon_protocol = 'http'

        if args.token_file:
            self.token = args.token_file.readline().rstrip('\r\n')
        if self.token is None:
            self.token = os.getenv('YDB_TOKEN')
            if self.token is not None:
                self.token = self.token.strip()
        if self.token is None:
            try:
                path = os.path.expanduser(os.path.join('~', '.ydb', 'token'))
                with open(path) as f:
                    self.token = f.readline().strip('\r\n')
            except Exception:
                pass
        self.domain = 1
        self.verbose = args.verbose
        self.quiet = args.quiet
        self.http_timeout = args.http_timeout
        self.cafile = args.cafile
        self.insecure = args.insecure
        self.http = args.http

    def add_host_access_options(self, parser, with_endpoint=True):
        parser.add_argument('--verbose', '-v', action='store_true', help='Be verbose during operation')
        parser.add_argument('--quiet', '-q', action='store_true', help="Don't show non-vital messages")
        g = parser.add_argument_group('Server access options')
        if with_endpoint:
            g.add_argument('--endpoint', '-e', metavar='[PROTOCOL://]HOST[:PORT]', type=str, required=True, action='append', help=ConnectionParams.ENDPOINT_HELP)
        g.add_argument('--grpc-port', type=int, default=2135, metavar='PORT', help='GRPC port to use for procedure invocation')
        g.add_argument('--mon-port', type=int, default=8765, metavar='PORT', help='HTTP monitoring port for viewer JSON access')
        g.add_argument('--mon-protocol', type=str, metavar='PROTOCOL', choices=('http', 'https'), help='HTTP monitoring protocol for viewer JSON access')
        g.add_argument('--token-file', type=FileType(encoding='ascii'), metavar='PATH', help='Path to token file')
        g.add_argument('--ca-file', metavar='PATH', dest='cafile', type=str, help='Path to a file containing the PEM encoding of the server root certificates for tls connections.')
        g.add_argument('--http', action='store_true', help='Use HTTP to connect to blob storage controller instead of GRPC')
        g.add_argument('--http-timeout', type=int, default=5, help='Timeout for blocking socket I/O operations during HTTP(s) queries')
        g.add_argument('--insecure', action='store_true', help='Allow insecure HTTPS fetching')


connection_params = ConnectionParams()


def set_connection_params_type(connection_params_type: type):
    global connection_params
    connection_params = connection_params_type()


def make_url(host, path, params):
    return connection_params.make_url(host, path, params)


get_pdisk_id = attrgetter('NodeId', 'PDiskId')
get_vslot_id = attrgetter('NodeId', 'PDiskId', 'VSlotId')
get_vslot_id_json = itemgetter('NodeId', 'PDiskId', 'VDiskSlotId')
get_vdisk_id = attrgetter('GroupId', 'GroupGeneration', 'FailRealmIdx', 'FailDomainIdx', 'VDiskIdx')
get_vdisk_id_json = itemgetter('GroupID', 'GroupGeneration', 'Ring', 'Domain', 'VDisk')
get_vdisk_id_short = attrgetter('FailRealmIdx', 'FailDomainIdx', 'VDiskIdx')


def get_vslot_extended_id(vslot):
    return *get_vslot_id(vslot.VSlotId), *get_vdisk_id(vslot)


class Location(typing.NamedTuple):
    dc: int
    room: int
    rack: int
    body: int
    node: int
    disk: int

    _levels = [10, 20, 30, 40, 254, 255]

    def __str__(self):
        return ','.join(str(x) if x is not None else '' for x in self)

    def __repr__(self):
        return 'Location(%s)' % ', '.join(map(str, self))

    def subs(self, begin, end):
        return Location._make(value if begin <= level < end else None for level, value in zip(Location._levels, self))

    @staticmethod
    def from_fail_domain(fdom):
        return Location(*map(fdom.get, Location._levels))

    @staticmethod
    def from_physical_location(loc):
        return Location.from_fail_domain(deserialize_fail_domain(loc))

    @staticmethod
    def from_location(location, node_id):
        return Location(dc=location.DataCenter, room=location.Module, rack=location.Rack, body=location.Unit, node=node_id, disk=None)


def inmemcache(name, params=[], cache_enable_param=None):
    def flatten_type(value):
        if isinstance(value, dict):
            return tuple(sorted(value.items()))
        elif isinstance(value, set):
            return tuple(sorted(value))
        else:
            return value

    def wrapper(func):
        sig = signature(func)

        @wraps(func)
        def wrapped(*args, **kwargs):
            a = sig.bind(*args, **kwargs)
            key = (name,) + tuple(flatten_type(a.arguments.get(p)) for p in params)
            if not cache_enable_param or a.arguments.get(cache_enable_param):
                return cache[key] if key in cache else cache.setdefault(key, func(*args, **kwargs))
            else:
                return func(*args, **kwargs)
        return wrapped
    return wrapper


class ConnectionError(Exception):
    pass


class QueryError(Exception):
    pass


class GroupSelectionError(Exception):
    pass


def query_random_host_with_retry(retries=5, explicit_host_param=None, http=False):
    def wrapper(func):
        sig = signature(func)

        @wraps(func)
        def wrapped(*args, **kwargs):
            explicit_host = None
            if explicit_host_param is not None:
                explicit_host = sig.bind(*args, **kwargs).arguments.get(explicit_host_param)

            allowed_hosts = {explicit_host} if explicit_host is not None else connection_params.hosts
            hosts_to_query = []

            try_index = 0
            while True:
                # regenerate host list if it got empty
                if not hosts_to_query:
                    hosts_to_query = list(allowed_hosts - bad_hosts) or list(allowed_hosts)
                    random.shuffle(hosts_to_query)

                host = hosts_to_query.pop()
                try:
                    return func(*args, **kwargs, host=host)
                except Exception as e:
                    try_index += 1
                    if isinstance(e, urllib.error.URLError):
                        bad_hosts.add(host)
                    if not connection_params.quiet:
                        print(f'WARNING: failed to fetch data from host {host} in {func.__name__}: {e}', file=sys.stderr)
                        if http and try_index == retries:
                            print('HINT: consider trying different protocol for endpoints when experiencing massive fetch failures from different hosts', file=sys.stderr)
                    if try_index == retries:
                        raise ConnectionError("Can't connect to specified addresses")

        return wrapped
    return wrapper


@inmemcache('fetch', ['path', 'params', 'explicit_host', 'fmt'], 'cache')
@query_random_host_with_retry(explicit_host_param='explicit_host', http=True)
def fetch(path, params={}, explicit_host=None, fmt='json', host=None, cache=True, method=None, data=None, content_type=None, accept=None):
    url = connection_params.make_url(host, path, params)
    if connection_params.verbose:
        print('INFO: fetching %s' % url, file=sys.stderr)
    request = urllib.request.Request(url, data=data, method=method)
    if connection_params.token and url.startswith('https://'):
        request.add_header('Authorization', 'OAuth %s' % connection_params.token)
    if content_type is not None:
        request.add_header('Content-Type', content_type)
    if accept is not None:
        request.add_header('Accept', accept)
    ctx = ssl.create_default_context(cafile=connection_params.cafile)
    if connection_params.insecure:
        ctx.check_hostname = False
        ctx.verify_mode = ssl.CERT_NONE
    stream = urllib.request.urlopen(request, timeout=connection_params.http_timeout, context=ctx)
    if fmt == 'json':
        return json.load(stream)
    elif fmt == 'raw':
        return stream.read()
    else:
        assert False, 'ERROR: invalid stream fmt specified: %s' % fmt


@query_random_host_with_retry()
def invoke_grpc(func, *params, host=None):
    options = [
        ('grpc.max_receive_message_length', 256 << 20),  # 256 MiB
    ]
    with grpc.insecure_channel('%s:%d' % (host, connection_params.grpc_port), options) as channel:
        if connection_params.verbose:
            p = ', '.join('<<< %s >>>' % text_format.MessageToString(param, as_one_line=True) for param in params)
            print('INFO: issuing %s(%s) @%s:%d' % (func, p, host, connection_params.grpc_port), file=sys.stderr)
        try:
            stub = kikimr_grpc.TGRpcServerStub(channel)
            res = getattr(stub, func)(*params)
            if connection_params.verbose:
                print('INFO: result <<< %s >>>' % text_format.MessageToString(res, as_one_line=True), file=sys.stderr)
            return res
        except Exception as e:
            if connection_params.verbose:
                print('ERROR: exception %s' % e, file=sys.stderr)
            raise ConnectionError("Can't connect to specified addresses by gRPC protocol")


def invoke_bsc_request(request):
    if connection_params.http:
        tablet_id = 72057594037932033
        data = request.SerializeToString()
        res = fetch('tablets/app', params=dict(TabletID=tablet_id, exec=1), fmt='raw', cache=False, method='POST',
                    data=data, content_type='application/x-protobuf', accept='application/x-protobuf')
        m = kikimr_bsconfig.TConfigResponse()
        m.MergeFromString(res)
        return m

    bs_request = kikimr_msgbus.TBlobStorageConfigRequest(Domain=connection_params.domain, Request=request)
    if connection_params.token is not None:
        bs_request.SecurityToken = connection_params.token
    bs_response = invoke_grpc('BlobStorageConfig', bs_request)
    if bs_response.Status != 1:
        # remove security token from error message
        bs_request.SecurityToken = ''
        request_s = text_format.MessageToString(bs_request, as_one_line=True)
        response_s = text_format.MessageToString(bs_response, as_one_line=True)
        raise QueryError('Failed to gRPC-query blob storage controller; request: %s; response: %s' % (request_s, response_s))
    return bs_response.BlobStorageConfigResponse


def cms_host_restart_request(user, host, reason, duration_usec, max_avail):
    cms_request = kikimr_msgbus.TCmsRequest()
    if connection_params.token is not None:
        cms_request.SecurityToken = connection_params.token
    cms_request.PermissionRequest.User = user
    action = cms_request.PermissionRequest.Actions.add()
    action.Type = kikimr_cms.TAction.EType.RESTART_SERVICES
    action.Host = host
    action.Services.append('storage')
    action.Duration = duration_usec
    cms_request.PermissionRequest.Reason = reason
    cms_request.PermissionRequest.Duration = duration_usec
    cms_request.PermissionRequest.AvailabilityMode = kikimr_cms.EAvailabilityMode.MODE_MAX_AVAILABILITY if max_avail else kikimr_cms.EAvailabilityMode.MODE_KEEP_AVAILABLE
    response = invoke_grpc('CmsRequest', cms_request)
    if response.Status.Code == kikimr_cms.TStatus.ECode.ALLOW:
        return None
    else:
        return '%s: %s' % (kikimr_cms.TStatus.ECode.Name(response.Status.Code), response.Status.Reason)


def create_bsc_request(args):
    request = kikimr_bsconfig.TConfigRequest(Rollback=args.dry_run)

    if hasattr(args, 'allow_unusable_pdisks') and args.allow_unusable_pdisks:
        request.AllowUnusableDisks = True
    if hasattr(args, 'ignore_degraded_group_check') and args.ignore_degraded_group_check:
        request.IgnoreDegradedGroupsChecks = True
    if hasattr(args, 'ignore_disintegrated_group_check') and args.ignore_disintegrated_group_check:
        request.IgnoreDisintegratedGroupsChecks = args.ignore_disintegrated_group_check
    if hasattr(args, 'ignore_failure_model_group_check') and args.ignore_failure_model_group_check:
        request.IgnoreGroupFailModelChecks = True
    if hasattr(args, 'ignore_vslot_quotas') and args.ignore_vslot_quotas:
        request.IgnoreVSlotQuotaCheck = True
    if hasattr(args, 'move_only_to_operational_pdisks') and args.move_only_to_operational_pdisks:
        request.SettleOnlyOnOperationalDisks = True

    return request


def create_wipe_request(args, vslot):
    request = create_bsc_request(args)
    cmd = request.Command.add().WipeVDisk
    cmd.VSlotId.NodeId = vslot.VSlotId.NodeId
    cmd.VSlotId.PDiskId = vslot.VSlotId.PDiskId
    cmd.VSlotId.VSlotId = vslot.VSlotId.VSlotId
    cmd.VDiskId.GroupID = vslot.GroupId
    cmd.VDiskId.GroupGeneration = vslot.GroupGeneration
    cmd.VDiskId.Ring = vslot.FailRealmIdx
    cmd.VDiskId.Domain = vslot.FailDomainIdx
    cmd.VDiskId.VDisk = vslot.VDiskIdx
    return request


def invoke_wipe_request(request):
    return invoke_bsc_request(request)


@inmemcache('base_config_and_storage_pools')
def fetch_base_config_and_storage_pools():
    request = kikimr_bsconfig.TConfigRequest(Rollback=True)
    request.Command.add().QueryBaseConfig.CopyFrom(kikimr_bsconfig.TQueryBaseConfig())
    request.Command.add().ReadStoragePool.BoxId = (1 << 64) - 1
    response = invoke_bsc_request(request)
    assert not response.Success
    assert len(response.Status) == 2
    assert response.Status[0].Success, 'QueryBaseConfig failed with error: %s' % response.Status[0].ErrorDescription
    assert response.Status[1].Success, 'ReadStoragePool failed with error: %s' % response.Status[1].ErrorDescription
    return dict(BaseConfig=response.Status[0].BaseConfig, StoragePools=response.Status[1].StoragePool)


def fetch_base_config():
    return fetch_base_config_and_storage_pools()['BaseConfig']


def fetch_storage_pools():
    return fetch_base_config_and_storage_pools()['StoragePools']


def fetch_node_mapping():
    base_config = fetch_base_config()
    return build_node_fqdn_maps(base_config)


def fetch_node_to_fqdn_map():
    return {node.NodeId: node.HostKey.Fqdn for node in fetch_base_config().Node}


def remove_and_pop_if_zero(m, key, value):
    x = m[key]
    x.remove(value)
    if not x:
        del m[key]


def map_fqdns(fqdns, node_ids, allowed_node_ids=None):
    # we have to query nodes and translate them to node ids
    _, fqdn_to_node_ids = fetch_node_mapping()
    while fqdns:
        name = fqdns.pop()
        fqdn, sep, port = name.partition(':')
        matching_ids = fqdn_to_node_ids.get(fqdn)
        if matching_ids and allowed_node_ids is not None:
            matching_ids = {
                node_id: port
                for node_id, port in matching_ids.items()
                if node_id in allowed_node_ids
            }
        if sep:
            matching_ids = {
                node_id: matching_port
                for node_id, matching_port in (matching_ids or dict()).items()
                if matching_port == int(port)
            }
        if not matching_ids:
            print('ERROR: FQDN %s not found' % (fqdn + sep + port), file=sys.stderr)
            sys.exit(1)
        elif len(matching_ids) > 1:
            print('ERROR: ambiguous FQDN %s matches nodes %s' % (name, ', '.join(map(str, sorted(matching_ids)))), file=sys.stderr)
            sys.exit(1)
        else:
            node_ids += matching_ids


def bytes_to_string(num, round, suffix):
    res_num = num / round
    if res_num < 10:
        s = f'{res_num:.2f}'
    elif res_num < 100:
        s = f'{res_num:.1f}'
    else:
        s = f'{res_num:.0f}'
    left, _, right = s.partition('.')
    if right == '00' or not right:
        right = ''
    else:
        right = f'.{right}'
    res = ''
    while left:
        subs, left = left[-3:], left[:-3]
        comma = "\'" if res else ''
        res = subs + comma + res
    return f'{res}{right}{suffix}'


def gib_string(num):
    return bytes_to_string(num, 1024 ** 3, '')


def bytes_string(num):
    if num > 1024 ** 5:
        return bytes_to_string(num, 1024 ** 5, ' PiB')
    if num > 1024 ** 4:
        return bytes_to_string(num, 1024 ** 4, ' TiB')
    if num > 1024 ** 3:
        return bytes_to_string(num, 1024 ** 3, ' GiB')
    if num > 1024 ** 2:
        return bytes_to_string(num, 1024 ** 2, ' MiB')
    if num > 1024:
        return bytes_to_string(num, 1024, ' kiB')
    return bytes_to_string(num, 1, '')


def convert_tristate_bool(tsb):
    if tsb == kikimr_bsconfig.ETriStateBool.kTrue:
        return True
    elif tsb == kikimr_bsconfig.ETriStateBool.kFalse:
        return False
    elif tsb == kikimr_bsconfig.ETriStateBool.kNotSet:
        return None
    print('ERROR: incorrect value for ETriStateBool', file=sys.stderr)
    sys.exit(1)


def deserialize_fail_domain(s):
    fmt = '=BI'
    step = struct.calcsize(fmt)
    res = {}
    for offset in range(0, len(s), step):
        key, value = struct.unpack_from(fmt, s, offset)
        res[key] = value
    return res


def pdisk_matches_storage_pool(pdisk, sp):
    if pdisk.BoxId != sp.BoxId:
        return False

    for pdisk_filter in sp.PDiskFilter:
        for prop in pdisk_filter.Property:
            if prop.HasField('Type'):
                if prop.Type != pdisk.Type:
                    break
            elif prop.HasField('SharedWithOs'):
                if prop.SharedWithOs != convert_tristate_bool(pdisk.SharedWithOs):
                    break
            elif prop.HasField('ReadCentric'):
                if prop.ReadCentric != convert_tristate_bool(pdisk.ReadCentric):
                    break
            elif prop.HasField('Kind'):
                if prop.Kind != pdisk.Kind:
                    break
            else:
                print('ERROR: unknown property in StoragePool filter', file=sys.stderr)
                sys.exit(1)
        else:
            return True

    return False


def select_groups(base_config, group_ids=None):
    if group_ids is not None:
        group_ids = set(group_ids)
    else:
        group_ids = set()

    known_groups = {
        group.GroupId
        for group in base_config.Group
        if is_dynamic_group(group.GroupId)
    }

    for group_id in group_ids:
        if not is_dynamic_group(group_id):
            raise Exception(False, 'Group {group_id} is static')
        if group_id not in known_groups:
            raise Exception('Unknown group with id {group_id}')

    if not group_ids:
        group_ids = known_groups
    return group_ids


def create_pdisk_map():
    base_config = fetch_base_config()
    node_to_location = {
        node.NodeId: Location.from_physical_location(node.PhysicalLocation)
        for node in base_config.Node
    }
    res = {}
    for pdisk in base_config.PDisk:
        location = node_to_location[pdisk.NodeId]._replace(node=pdisk.NodeId, disk=pdisk.PDiskId)
        res[location] = pdisk
    return res


def vslots_of_group(group, vslot_map):
    return map(vslot_map.__getitem__, map(get_vslot_id, group.VSlotId))


def build_group_slot_size_map(base_config, vslot_map):
    return {
        group.GroupId: max(vslot.VDiskMetrics.AllocatedSize for vslot in vslots_of_group(group, vslot_map))
        for group in base_config.Group
    }


def build_group_map(base_config):
    group_map = {
        group.GroupId: group
        for group in base_config.Group
        if is_dynamic_group(group.GroupId)
    }
    return group_map


def build_node_fqdn_map(base_config):
    node_fqdn_map = {
        node.NodeId: node.HostKey.Fqdn
        for node in base_config.Node
    }
    return node_fqdn_map


def build_node_fqdn_maps(base_config):
    node_id_to_host = {}
    host_to_node_id = {}
    for node in base_config.Node:
        node_id_to_host[node.NodeId] = (node.HostKey.Fqdn, node.HostKey.IcPort)
        host_to_node_id.setdefault(node.HostKey.Fqdn, {})[node.NodeId] = node.HostKey.IcPort
    return node_id_to_host, host_to_node_id


def build_pdisk_map(base_config):
    pdisk_map = {
        get_pdisk_id(pdisk): pdisk
        for pdisk in base_config.PDisk
    }
    return pdisk_map


def build_pdisk_static_slots_map(base_config):
    pdisk_static_slots_map = {
        get_pdisk_id(pdisk): pdisk.NumStaticSlots
        for pdisk in base_config.PDisk
    }
    return pdisk_static_slots_map


def build_pdisk_usage_map(base_config, count_donors=False, storage_pool=None):
    pdisk_usage_map = {}

    for pdisk in base_config.PDisk:
        if storage_pool is not None and not pdisk_matches_storage_pool(pdisk, storage_pool):
            continue
        pdisk_id = get_pdisk_id(pdisk)
        pdisk_usage_map[pdisk_id] = pdisk.NumStaticSlots

    for vslot in base_config.VSlot:
        pdisk_id = get_pdisk_id(vslot.VSlotId)
        if pdisk_id not in pdisk_usage_map:
            continue
        pdisk_usage_map[pdisk_id] += 1
        for donor in vslot.Donors if count_donors else []:
            donor_pdisk_id = get_pdisk_id(donor.VSlotId)
            pdisk_usage_map[donor_pdisk_id] += 1

    return pdisk_usage_map


def build_storage_pools_map(storage_pools):
    storage_pools_map = {
        (sp.BoxId, sp.StoragePoolId): sp
        for sp in storage_pools
    }
    return storage_pools_map


def build_storage_pool_groups_map(base_config, group_ids):
    storage_pool_groups_map = defaultdict(list)
    for group in base_config.Group:
        if group.GroupId in group_ids:
            storage_pool_groups_map[group.BoxId, group.StoragePoolId].append(group)

    known_groups = {
        group.GroupId
        for group in base_config.Group
    }

    for group_id in group_ids:
        if group_id not in known_groups:
            raise Exception('Unknown group with id %u' % group_id)

    return storage_pool_groups_map


def build_storage_pool_names_map(storage_pools):
    storage_pool_names_map = {
        (sp.BoxId, sp.StoragePoolId): sp.Name
        for sp in storage_pools
    }
    return storage_pool_names_map


def build_vslot_map(base_config):
    vslot_map = {
        get_vslot_id(vslot.VSlotId): vslot
        for vslot in base_config.VSlot
    }
    return vslot_map


def message_to_string(m):
    return text_format.MessageToString(m, as_one_line=True)


def add_pdisk_select_options(parser, specification=None):
    types = kikimr_bsconfig.EPDiskType.keys()
    name = 'PDisk selection options'
    if specification is not None:
        name += ' for ' + specification
    g = parser.add_argument_group(name)
    g.add_argument('--node-id', type=int, nargs='+', metavar='NODE', help='filter only PDisks on a node(s) with specific number')
    g.add_argument('--fqdn', type=str, nargs='+', metavar='FQDN', help='filter only PDisks on a node with specific FQDN(s)')
    g.add_argument('--pdisk-id', type=int, nargs='+', metavar='PDISK', help='filter only PDisks with specific id')
    g.add_argument('--path', type=str, metavar='PATH', help='filter only PDisks with a specific path on a system')
    g.add_argument('--type', type=str, choices=types, metavar='TYPE', help='filter only PDisks with a specific media type')


def add_format_options(parser, formats: list, default=None):
    help_lines = ['Output format. Available options:']
    for format_type, description_lines in formats:
        help_lines.append('  ' + format_type)
        for line in description_lines:
            help_lines.append('    ' + line)
    help = '\n'.join(help_lines)
    choices = [format_type for format_type, _ in formats]
    parser.add_argument('--format', type=str, choices=choices, default=default, help=help)


def add_basic_format_options(parser):
    basic_formats = [
        ('pretty', ['Human readable output']),
        ('json', ['Output in json format'])
    ]
    add_format_options(parser, basic_formats, default='pretty')


def get_selected_pdisks(args, base_config):
    node_id_to_host = {
        node.NodeId: node.HostKey.Fqdn
        for node in base_config.Node
    }
    return {
        (pdisk.NodeId, pdisk.PDiskId)
        for pdisk in base_config.PDisk
        if args.node_id is None or pdisk.NodeId in args.node_id
        if args.fqdn is None or any(fnmatch.fnmatchcase(node_id_to_host[pdisk.NodeId], fqdn) for fqdn in args.fqdn)
        if args.pdisk_id is None or pdisk.PDiskId in args.pdisk_id
        if args.path in [None, pdisk.Path]
        if args.type in [None, kikimr_bsconfig.EPDiskType.Name(pdisk.Type)]
    }


def fetch_json_info(entity, nodes=None, enums=1):
    merge = None
    if entity == 'pdiskinfo':
        section, keycols = 'PDiskStateInfo', ['NodeId', 'PDiskId']
    elif entity == 'sysinfo':
        section, keycols = 'SystemStateInfo', ['NodeId']
    elif entity == 'vdiskinfo':
        section, keycols = 'VDiskStateInfo', ['NodeId', 'PDiskId', 'VDiskSlotId']

        def merge(x, y):
            return max([x, y], key=lambda x: x.get('GroupGeneration', 0))

    elif entity == 'tabletinfo':
        section, keycols = 'TabletStateInfo', ['TabletId']
    elif entity == 'bsgroupinfo':
        section, keycols = 'BSGroupStateInfo', ['GroupID']

        def merge(x, y):
            if x.get('GroupGeneration', 0) > y.get('GroupGeneration', 0):
                return x
            if y.get('GroupGeneration', 0) > x.get('GroupGeneration', 0):
                return y
            if x.get('VDiskIds', []):
                return x
            return y
    else:
        assert False
    res = {}
    key_getter = itemgetter(*keycols)
    max_nodes_at_once = 128
    if nodes is None:
        nodes = map(attrgetter('NodeId'), fetch_base_config().Node)
    remaining_retry_count = {node_id: 5 for node_id in nodes}
    missing_node_ids = set()
    while remaining_retry_count:
        node_ids = sorted(remaining_retry_count, key=lambda x: (-remaining_retry_count[x], x))[:max_nodes_at_once]
        node_id = ','.join(map(str, node_ids))
        for node_id_str, j in fetch('viewer/json/%s' % entity, dict(enums=enums, merge=0, node_id=node_id), cache=False).items():
            if section in j:
                remaining_retry_count.pop(int(node_id_str), None)
                for item in j[section]:
                    item['NodeId'] = int(node_id_str)
                    key = key_getter(item)
                    if not merge and key in res:
                        print('non-callable merge entity=%s key=%s item=%s prev=%s' % (entity, key, item, res[key]), file=sys.stderr)
                    res[key] = merge(res[key], item) if key in res else item
        for node_id in node_ids:
            if node_id in remaining_retry_count:
                remaining_retry_count[node_id] -= 1
                if not remaining_retry_count[node_id]:
                    del remaining_retry_count[node_id]
                    missing_node_ids.add(node_id)
    if missing_node_ids:
        print('WARNING: missing NodeId# %s' % ', '.join(map(str, sorted(missing_node_ids))), file=sys.stderr)
    return res


def fetch_node_mon_map(nodes=None):
    return {
        node_id: sysinfo['Host'] + ep['Address']
        for node_id, sysinfo in fetch_json_info('sysinfo', nodes).items()
        for ep in sysinfo.get('Endpoints', [])
        if ep['Name'] == 'http-mon'
    }


def get_vslots_by_vdisk_ids(base_config, vdisk_ids):
    vdisk_vslot_map = {}
    for v in base_config.VSlot:
        vdisk_vslot_map['[%08x:_:%u:%u:%u]' % (v.GroupId, v.FailRealmIdx, v.FailDomainIdx, v.VDiskIdx)] = v
        vdisk_vslot_map['[%08x:%u:%u:%u:%u]' % (v.GroupId, v.GroupGeneration, v.FailRealmIdx, v.FailDomainIdx, v.VDiskIdx)] = v

    res = []
    for string in vdisk_ids:
        for vdisk_id in string.split():
            if vdisk_id not in vdisk_vslot_map:
                raise Exception('VDisk with id %s not found' % vdisk_id)
            vslot = vdisk_vslot_map[vdisk_id]
            res.append(vslot)
    return res


def fetch_vdisk_status(hostname):
    res = []
    try:
        j = fetch('viewer/json/vdiskinfo', dict(enums=1, node_id=0), hostname, cache=False)
    except Exception:
        return []
    for v in j.get('VDiskStateInfo', []):
        try:
            res.append((hostname, *get_vslot_id_json(v), *get_vdisk_id_json(v['VDiskId']), v['VDiskState'], v['Replicated']))
        except KeyError:
            pass
    return res


def filter_healthy_groups(groups, node_mon_map, base_config, vslot_map):
    res = {
        group.GroupId: len(group.VSlotId)
        for group in base_config.Group
        if group.GroupId in groups
        if all(vslot.Status == 'READY' for vslot in vslots_of_group(group, vslot_map))
    }
    check_set = {
        (*vslot_id, *attrgetter('GroupId', 'FailRealmIdx', 'FailDomainIdx', 'VDiskIdx')(vslot))
        for vslot_id, vslot in vslot_map.items()
        if vslot.GroupId in res
    }
    for vdisk_id, j in fetch_json_info('vdiskinfo', {node_id for node_id, _, _, _, _, _, _ in check_set}).items():
        if j.get('Replicated') and j.get('VDiskState') == 'OK':
            check_item = *vdisk_id, *itemgetter('GroupID', 'Ring', 'Domain', 'VDisk')(j['VDiskId'])
            if check_item in check_set:
                check_set.remove(check_item)
                res[j['VDiskId']['GroupID']] -= 1
    return {group_id for group_id, count in res.items() if not count}


def add_host_access_options(parser):
    connection_params.add_host_access_options(parser)


def add_vdisk_ids_option(g, required=False):
    g.add_argument('--vdisk-ids', type=str, nargs='+', required=required, help='Space separated list of vdisk ids in format [GroupId:_:FailRealm:FailDomain:VDiskIdx]')


def add_pdisk_ids_option(p, required=False):
    p.add_argument('--pdisk-ids', type=str, nargs='+', required=required, help='Space separated list of pdisk ids in format [NodeId:PDiskId]')


def add_group_ids_option(p, required=False):
    p.add_argument('--group-ids', type=int, nargs='+', action='append', required=required, help='Space separated list of group ids')


def add_allow_unusable_pdisks_option(p):
    p.add_argument('--allow-unusable-pdisks', action='store_true', help='Allow unusable PDisks to stay in place while replacing other ones')


def add_ignore_degraded_group_check_option(p):
    p.add_argument('--ignore-degraded-group-check', action='store_true', help='Ignore results of DEGRADED group checks')


def add_ignore_disintegrated_group_check_option(p):
    p.add_argument('--ignore-disintegrated-group-check', action='store_true', help='Ignore results of DISINTEGRATED group checks')


def add_ignore_failure_model_group_check_option(p):
    p.add_argument('--ignore-failure-model-group-check', action='store_true', help='Ignore results of failure model group checks')


def add_ignore_vslot_quotas_option(p):
    p.add_argument('--ignore-vslot-quotas', action='store_true', help='Ignore results of VSlot quota checks')


def apply_args(args):
    connection_params.apply_args(args)


def flush_cache():
    cache.clear()


def print_json_result(status: str, description: str = None, file=sys.stdout):
    d = {'status': status}
    if description is not None:
        d['description'] = description
    print(json.dumps(d), file=file)


def print_result(format: str, status: str, description: str = None, file=sys.stderr):
    if format == 'json':
        print_json_result(status, description)
    else:
        if description is not None:
            print('{0}, {1}'.format(status, description), file=file)
        else:
            print(status, file=file)


def print_request_result(args, request, response):
    success = is_successful_bsc_response(response)
    error_reason = 'Request has failed: \n{0}\n{1}\n'.format(request, response)
    print_status_if_verbose(args, success, error_reason)


def print_status_if_verbose(args, success, error_reason):
    format = getattr(args, 'format', 'pretty')
    verbose = getattr(args, 'verbose', False)
    if success:
        print_result(format, 'success')
    else:
        if verbose:
            print_result(format, 'error', error_reason)
        else:
            print_result(format, 'error', 'add --verbose for more info')


def print_status_if_not_quiet(args, success, error_reason):
    format = getattr(args, 'format', 'pretty')
    quiet = getattr(args, 'quiet', False)
    if success:
        print_result(format, 'success')
    else:
        if not quiet:
            print_result(format, 'error', error_reason)


def print_status(args, success, error_reason):
    print_status_if_not_quiet(args, success, error_reason)


def print_if_verbose(args, message, file=sys.stderr):
    verbose = getattr(args, 'verbose', False)
    format = getattr(args, 'format', 'pretty')
    if verbose:
        print_result(format, status=message, description=None, file=file)


def print_if_not_quiet(args, message, file=sys.stderr):
    quiet = getattr(args, 'quiet', False)
    format = getattr(args, 'format', 'pretty')
    if not quiet:
        print_result(format, status=message, description=None, file=file)


def is_dynamic_group(groupId):
    return groupId >= 0x80000000


def is_successful_bsc_response(response):
    return response.Success or 'transaction rollback' in response.ErrorDescription



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/core/protos/table_stats.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!ydb/core/protos/table_stats.proto\x12\x11NKikimrTableStats\".\n\x10THistogramBucket\x12\x0b\n\x03Key\x18\x01 \x01(\x0c\x12\r\n\x05Value\x18\x02 \x01(\x04\"B\n\nTHistogram\x12\x34\n\x07\x42uckets\x18\x01 \x03(\x0b\x32#.NKikimrTableStats.THistogramBucket\"\x9e\x05\n\x0bTTableStats\x12\x10\n\x08\x44\x61taSize\x18\x01 \x01(\x04\x12\x10\n\x08RowCount\x18\x02 \x01(\x04\x12\x11\n\tIndexSize\x18\x03 \x01(\x04\x12\x11\n\tInMemSize\x18\x04 \x01(\x04\x12\x16\n\x0eLastAccessTime\x18\x05 \x01(\x04\x12\x16\n\x0eLastUpdateTime\x18\x06 \x01(\x04\x12\x38\n\x11RowCountHistogram\x18\x07 \x01(\x0b\x32\x1d.NKikimrTableStats.THistogram\x12\x38\n\x11\x44\x61taSizeHistogram\x18\x08 \x01(\x0b\x32\x1d.NKikimrTableStats.THistogram\x12\x1c\n\x14ImmediateTxCompleted\x18\t \x01(\x04\x12\x1a\n\x12PlannedTxCompleted\x18\n \x01(\x04\x12\x1c\n\x14TxRejectedByOverload\x18\x0b \x01(\x04\x12\x19\n\x11TxRejectedBySpace\x18\x0c \x01(\x04\x12\x19\n\x11TxCompleteLagMsec\x18\r \x01(\x04\x12\x17\n\x0fInFlightTxCount\x18\x0e \x01(\x04\x12\x12\n\nRowUpdates\x18\x14 \x01(\x04\x12\x12\n\nRowDeletes\x18\x15 \x01(\x04\x12\x10\n\x08RowReads\x18\x16 \x01(\x04\x12\x12\n\nRangeReads\x18\x17 \x01(\x04\x12\x15\n\rRangeReadRows\x18\x19 \x01(\x04\x12\x11\n\tPartCount\x18\x18 \x01(\x04\x12\x36\n\x0fKeyAccessSample\x18\x1a \x01(\x0b\x32\x1d.NKikimrTableStats.THistogram\x12\x14\n\x0cSearchHeight\x18\x1b \x01(\x04\x12\x1c\n\x14LastFullCompactionTs\x18\x1c \x01(\x04\x12\x16\n\x0eHasLoanedParts\x18\x1d \x01(\x08\x42\x18\n\x16ru.yandex.kikimr.proto')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.core.protos.table_stats_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026ru.yandex.kikimr.proto'
  _THISTOGRAMBUCKET._serialized_start=56
  _THISTOGRAMBUCKET._serialized_end=102
  _THISTOGRAM._serialized_start=104
  _THISTOGRAM._serialized_end=170
  _TTABLESTATS._serialized_start=173
  _TTABLESTATS._serialized_end=843
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/core/protos/scheme_log.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb.core.protos import flat_scheme_op_pb2 as ydb_dot_core_dot_protos_dot_flat__scheme__op__pb2
from ydb.core.protos import type_info_pb2 as ydb_dot_core_dot_protos_dot_type__info__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n ydb/core/protos/scheme_log.proto\x12\x11NTabletFlatScheme\x1a$ydb/core/protos/flat_scheme_op.proto\x1a\x1fydb/core/protos/type_info.proto\"\xa1\x08\n\x0cTAlterRecord\x12=\n\tDeltaType\x18\x01 \x02(\x0e\x32*.NTabletFlatScheme.TAlterRecord.EDeltaType\x12\x0f\n\x07TableId\x18\x02 \x01(\r\x12\x11\n\tTableName\x18\x03 \x01(\t\x12\x10\n\x08\x43olumnId\x18\x04 \x01(\r\x12\x10\n\x08\x46\x61milyId\x18\x07 \x01(\r\x12\x0e\n\x06RoomId\x18\x08 \x01(\r\x12\r\n\x05\x41nnex\x18\x10 \x01(\r\x12\x16\n\x0b\x42yKeyFilter\x18\x11 \x01(\r:\x01\x30\x12\x19\n\x11\x45raseCacheEnabled\x18\x14 \x01(\x08\x12\x19\n\x11\x45raseCacheMinRows\x18\x15 \x01(\r\x12\x1a\n\x12\x45raseCacheMaxBytes\x18\x16 \x01(\r\x12\x19\n\nColdBorrow\x18\x17 \x01(\x08:\x05\x66\x61lse\x12\x0c\n\x04Main\x18\t \x01(\r\x12\r\n\x05\x42lobs\x18\x0f \x01(\r\x12\r\n\x05Outer\x18\x13 \x01(\r\x12\r\n\x05\x43odec\x18\x0b \x01(\r\x12\x10\n\x08InMemory\x18\x0c \x01(\x08\x12\r\n\x05\x43\x61\x63he\x18\r \x01(\r\x12\r\n\x05Large\x18\x0e \x01(\r\x12\r\n\x05Small\x18\x12 \x01(\r\x12\x12\n\nColumnName\x18\x05 \x01(\t\x12\x12\n\nColumnType\x18\x06 \x01(\r\x12/\n\x0e\x43olumnTypeInfo\x18\x19 \x01(\x0b\x32\x17.NKikimrProto.TTypeInfo\x12\x0f\n\x07\x44\x65\x66\x61ult\x18\n \x01(\x0c\x12\x16\n\x07NotNull\x18\x18 \x01(\x08:\x05\x66\x61lse\x12\x19\n\x11\x45xecutorCacheSize\x18\x64 \x01(\x04\x12<\n\x10\x43ompactionPolicy\x18\x65 \x01(\x0b\x32\".NKikimrSchemeOp.TCompactionPolicy\x12 \n\x18\x45xecutorAllowLogBatching\x18\x66 \x01(\x08\x12\x1e\n\x16\x45xecutorLogFlushPeriod\x18g \x01(\x04\x12\x1c\n\x14\x45xecutorLimitInFlyTx\x18h \x01(\r\x12\x1f\n\x17\x45xecutorResourceProfile\x18i \x01(\t\x12#\n\x1b\x45xecutorLogFastCommitTactic\x18j \x01(\x08\"\xea\x01\n\nEDeltaType\x12\x0c\n\x08\x41\x64\x64Table\x10\x01\x12\r\n\tDropTable\x10\x02\x12\r\n\tAddColumn\x10\x03\x12\x0e\n\nDropColumn\x10\x04\x12\x12\n\x0e\x41\x64\x64\x43olumnToKey\x10\x05\x12\x15\n\x11\x41\x64\x64\x43olumnToFamily\x10\x06\x12\r\n\tAddFamily\x10\x07\x12\x16\n\x12UpdateExecutorInfo\x10\x08\x12\x17\n\x13SetCompactionPolicy\x10\t\x12\x0b\n\x07SetRoom\x10\n\x12\r\n\tSetFamily\x10\x0b\x12\x0b\n\x07SetRedo\x10\x0c\x12\x0c\n\x08SetTable\x10\r\"@\n\x0eTSchemeChanges\x12.\n\x05\x44\x65lta\x18\x01 \x03(\x0b\x32\x1f.NTabletFlatScheme.TAlterRecordB\x18\n\x16ru.yandex.kikimr.proto')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.core.protos.scheme_log_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026ru.yandex.kikimr.proto'
  _TALTERRECORD._serialized_start=127
  _TALTERRECORD._serialized_end=1184
  _TALTERRECORD_EDELTATYPE._serialized_start=950
  _TALTERRECORD_EDELTATYPE._serialized_end=1184
  _TSCHEMECHANGES._serialized_start=1186
  _TSCHEMECHANGES._serialized_end=1250
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/core/protos/console_base.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb.public.api.protos import ydb_status_codes_pb2 as ydb_dot_public_dot_api_dot_protos_dot_ydb__status__codes__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"ydb/core/protos/console_base.proto\x12\x0eNKikimrConsole\x1a,ydb/public/api/protos/ydb_status_codes.proto\"B\n\x07TStatus\x12\'\n\x04\x43ode\x18\x01 \x01(\x0e\x32\x19.Ydb.StatusIds.StatusCode\x12\x0e\n\x06Reason\x18\x02 \x01(\tB\x18\n\x16ru.yandex.kikimr.proto')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.core.protos.console_base_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026ru.yandex.kikimr.proto'
  _TSTATUS._serialized_start=100
  _TSTATUS._serialized_end=166
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/public/api/protos/draft/ydb_logstore.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb.public.api.protos import ydb_operation_pb2 as ydb_dot_public_dot_api_dot_protos_dot_ydb__operation__pb2
from ydb.public.api.protos import ydb_value_pb2 as ydb_dot_public_dot_api_dot_protos_dot_ydb__value__pb2
from ydb.public.api.protos import ydb_scheme_pb2 as ydb_dot_public_dot_api_dot_protos_dot_ydb__scheme__pb2
from ydb.public.api.protos import ydb_table_pb2 as ydb_dot_public_dot_api_dot_protos_dot_ydb__table__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.ydb/public/api/protos/draft/ydb_logstore.proto\x12\x0cYdb.LogStore\x1a)ydb/public/api/protos/ydb_operation.proto\x1a%ydb/public/api/protos/ydb_value.proto\x1a&ydb/public/api/protos/ydb_scheme.proto\x1a%ydb/public/api/protos/ydb_table.proto\x1a\x1bgoogle/protobuf/empty.proto\"\xb4\x01\n\x0b\x43ompression\x12:\n\x11\x63ompression_codec\x18\x01 \x01(\x0e\x32\x1f.Ydb.LogStore.Compression.Codec\x12\x19\n\x11\x63ompression_level\x18\x02 \x01(\x05\"N\n\x05\x43odec\x12\x15\n\x11\x43ODEC_UNSPECIFIED\x10\x00\x12\x0f\n\x0b\x43ODEC_PLAIN\x10\x01\x12\r\n\tCODEC_LZ4\x10\x02\x12\x0e\n\nCODEC_ZSTD\x10\x03\"3\n\nColumnMeta\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x17\n\x04type\x18\x02 \x01(\x0b\x32\t.Ydb.Type\"\x80\x01\n\x06Schema\x12)\n\x07\x63olumns\x18\x01 \x03(\x0b\x32\x18.Ydb.LogStore.ColumnMeta\x12\x13\n\x0bprimary_key\x18\x02 \x03(\t\x12\x36\n\x13\x64\x65\x66\x61ult_compression\x18\x03 \x01(\x0b\x32\x19.Ydb.LogStore.Compression\"B\n\x0cSchemaPreset\x12\x0c\n\x04name\x18\x01 \x01(\t\x12$\n\x06schema\x18\x02 \x01(\x0b\x32\x14.Ydb.LogStore.Schema\"J\n\nTierConfig\x12\x0c\n\x04name\x18\x01 \x01(\t\x12.\n\x0b\x63ompression\x18\x02 \x01(\x0b\x32\x19.Ydb.LogStore.Compression\">\n\x04Tier\x12\x0c\n\x04name\x18\x01 \x01(\t\x12(\n\x08\x65viction\x18\x02 \x01(\x0b\x32\x16.Ydb.Table.TtlSettings\"9\n\x0fTieringSettings\x12\x17\n\ntiering_id\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\r\n\x0b_tiering_id\"\xaa\x01\n\x15\x43reateLogStoreRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x14\n\x0cshards_count\x18\x03 \x01(\r\x12\x32\n\x0eschema_presets\x18\x04 \x03(\x0b\x32\x1a.Ydb.LogStore.SchemaPreset\"F\n\x16\x43reateLogStoreResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"b\n\x17\x44\x65scribeLogStoreRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\"\xac\x01\n\x16\x44\x65scribeLogStoreResult\x12\x1f\n\x04self\x18\x01 \x01(\x0b\x32\x11.Ydb.Scheme.Entry\x12\x14\n\x0cshards_count\x18\x02 \x01(\r\x12\x32\n\x0eschema_presets\x18\x03 \x03(\x0b\x32\x1a.Ydb.LogStore.SchemaPreset\x12\'\n\x05tiers\x18\x04 \x03(\x0b\x32\x18.Ydb.LogStore.TierConfig\"H\n\x18\x44\x65scribeLogStoreResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"^\n\x13\x44ropLogStoreRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\"D\n\x14\x44ropLogStoreResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\xac\x02\n\x14\x41lterLogStoreRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x37\n\x11set_schema_preset\x18\x03 \x01(\x0b\x32\x1a.Ydb.LogStore.SchemaPresetH\x00\x12\x1c\n\x12\x64rop_schema_preset\x18\x04 \x01(\tH\x00\x12,\n\x08set_tier\x18\x05 \x01(\x0b\x32\x18.Ydb.LogStore.TierConfigH\x01\x12\x13\n\tdrop_tier\x18\x06 \x01(\tH\x01\x42\x1c\n\x1aschema_preset_modificationB\x13\n\x11tier_modification\"E\n\x15\x41lterLogStoreResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\xc2\x03\n\x15\x43reateLogTableRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x1c\n\x12schema_preset_name\x18\x03 \x01(\tH\x00\x12&\n\x06schema\x18\x04 \x01(\x0b\x32\x14.Ydb.LogStore.SchemaH\x00\x12.\n\x0cttl_settings\x18\x05 \x01(\x0b\x32\x16.Ydb.Table.TtlSettingsH\x01\x12\x39\n\x10tiering_settings\x18\x06 \x01(\x0b\x32\x1d.Ydb.LogStore.TieringSettingsH\x01\x12\x14\n\x0cshards_count\x18\x07 \x01(\r\x12\x18\n\x10sharding_columns\x18\x08 \x03(\t\x12\x35\n\rsharding_type\x18\t \x01(\x0e\x32\x1e.Ydb.LogStore.ShardingHashType\x12\x1b\n\x13\x61\x63tive_shards_count\x18\n \x01(\rB\x16\n\x14schema_specificationB\x13\n\x11ttl_specification\"F\n\x16\x43reateLogTableResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"b\n\x17\x44\x65scribeLogTableRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\"\xff\x02\n\x16\x44\x65scribeLogTableResult\x12\x1f\n\x04self\x18\x01 \x01(\x0b\x32\x11.Ydb.Scheme.Entry\x12\x1a\n\x12schema_preset_name\x18\x02 \x01(\t\x12$\n\x06schema\x18\x03 \x01(\x0b\x32\x14.Ydb.LogStore.Schema\x12.\n\x0cttl_settings\x18\x05 \x01(\x0b\x32\x16.Ydb.Table.TtlSettingsH\x00\x12\x39\n\x10tiering_settings\x18\x04 \x01(\x0b\x32\x1d.Ydb.LogStore.TieringSettingsH\x00\x12\x14\n\x0cshards_count\x18\x06 \x01(\r\x12\x18\n\x10sharding_columns\x18\x07 \x03(\t\x12\x35\n\rsharding_type\x18\x08 \x01(\x0e\x32\x1e.Ydb.LogStore.ShardingHashType\x12\x1b\n\x13\x61\x63tive_shards_count\x18\t \x01(\rB\x13\n\x11ttl_specification\"H\n\x18\x44\x65scribeLogTableResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"^\n\x13\x44ropLogTableRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\"D\n\x14\x44ropLogTableResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x95\x02\n\x14\x41lterLogTableRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x33\n\x11\x64rop_ttl_settings\x18\x03 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x00\x12\x32\n\x10set_ttl_settings\x18\x04 \x01(\x0b\x32\x16.Ydb.Table.TtlSettingsH\x00\x12=\n\x14set_tiering_settings\x18\x05 \x01(\x0b\x32\x1d.Ydb.LogStore.TieringSettingsH\x00\x42\x0c\n\nttl_action\"E\n\x15\x41lterLogTableResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation*a\n\x10ShardingHashType\x12\x19\n\x15HASH_TYPE_UNSPECIFIED\x10\x00\x12\x16\n\x12HASH_TYPE_MODULO_N\x10\x01\x12\x1a\n\x16HASH_TYPE_LOGS_SPECIAL\x10\x02\x42,\n\x17\x63om.yandex.ydb.logstoreB\x0eLogStoreProtos\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.public.api.protos.draft.ydb_logstore_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027com.yandex.ydb.logstoreB\016LogStoreProtos\370\001\001'
  _SHARDINGHASHTYPE._serialized_start=3624
  _SHARDINGHASHTYPE._serialized_end=3721
  _COMPRESSION._serialized_start=255
  _COMPRESSION._serialized_end=435
  _COMPRESSION_CODEC._serialized_start=357
  _COMPRESSION_CODEC._serialized_end=435
  _COLUMNMETA._serialized_start=437
  _COLUMNMETA._serialized_end=488
  _SCHEMA._serialized_start=491
  _SCHEMA._serialized_end=619
  _SCHEMAPRESET._serialized_start=621
  _SCHEMAPRESET._serialized_end=687
  _TIERCONFIG._serialized_start=689
  _TIERCONFIG._serialized_end=763
  _TIER._serialized_start=765
  _TIER._serialized_end=827
  _TIERINGSETTINGS._serialized_start=829
  _TIERINGSETTINGS._serialized_end=886
  _CREATELOGSTOREREQUEST._serialized_start=889
  _CREATELOGSTOREREQUEST._serialized_end=1059
  _CREATELOGSTORERESPONSE._serialized_start=1061
  _CREATELOGSTORERESPONSE._serialized_end=1131
  _DESCRIBELOGSTOREREQUEST._serialized_start=1133
  _DESCRIBELOGSTOREREQUEST._serialized_end=1231
  _DESCRIBELOGSTORERESULT._serialized_start=1234
  _DESCRIBELOGSTORERESULT._serialized_end=1406
  _DESCRIBELOGSTORERESPONSE._serialized_start=1408
  _DESCRIBELOGSTORERESPONSE._serialized_end=1480
  _DROPLOGSTOREREQUEST._serialized_start=1482
  _DROPLOGSTOREREQUEST._serialized_end=1576
  _DROPLOGSTORERESPONSE._serialized_start=1578
  _DROPLOGSTORERESPONSE._serialized_end=1646
  _ALTERLOGSTOREREQUEST._serialized_start=1649
  _ALTERLOGSTOREREQUEST._serialized_end=1949
  _ALTERLOGSTORERESPONSE._serialized_start=1951
  _ALTERLOGSTORERESPONSE._serialized_end=2020
  _CREATELOGTABLEREQUEST._serialized_start=2023
  _CREATELOGTABLEREQUEST._serialized_end=2473
  _CREATELOGTABLERESPONSE._serialized_start=2475
  _CREATELOGTABLERESPONSE._serialized_end=2545
  _DESCRIBELOGTABLEREQUEST._serialized_start=2547
  _DESCRIBELOGTABLEREQUEST._serialized_end=2645
  _DESCRIBELOGTABLERESULT._serialized_start=2648
  _DESCRIBELOGTABLERESULT._serialized_end=3031
  _DESCRIBELOGTABLERESPONSE._serialized_start=3033
  _DESCRIBELOGTABLERESPONSE._serialized_end=3105
  _DROPLOGTABLEREQUEST._serialized_start=3107
  _DROPLOGTABLEREQUEST._serialized_end=3201
  _DROPLOGTABLERESPONSE._serialized_start=3203
  _DROPLOGTABLERESPONSE._serialized_end=3271
  _ALTERLOGTABLEREQUEST._serialized_start=3274
  _ALTERLOGTABLEREQUEST._serialized_end=3551
  _ALTERLOGTABLERESPONSE._serialized_start=3553
  _ALTERLOGTABLERESPONSE._serialized_end=3622
# @@protoc_insertion_point(module_scope)



import ydb.apps.dstool.lib.common as common
import sys

description = 'Remove disk from storage by serial number'


def add_options(p):
    p.add_argument('--serial', type=str, required=True, help="Disk serial number")
    common.add_basic_format_options(p)


def create_request(args):
    request = common.kikimr_bsconfig.TConfigRequest(Rollback=args.dry_run)
    cmd = request.Command.add().RemoveDriveSerial
    cmd.Serial = args.serial
    return request


def perform_request(request):
    return common.invoke_bsc_request(request)


def is_successful_response(response):
    return common.is_successful_bsc_response(response)


def do(args):
    request = create_request(args)
    response = perform_request(request)
    common.print_request_result(args, request, response)
    if not is_successful_response(response):
        fail_reasons = set([status.FailReason for status in response.Status])
        if len(fail_reasons) == 1:
            sys.exit(100 + list(fail_reasons)[0])
        else:
            sys.exit(1)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/core/protos/follower_group.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$ydb/core/protos/follower_group.proto\x12\x0bNKikimrHive\"\xb0\x02\n\x0eTFollowerGroup\x12\x15\n\rFollowerCount\x18\x01 \x01(\r\x12\x1c\n\x14\x41llowLeaderPromotion\x18\x02 \x01(\x08\x12\x17\n\x0f\x41llowClientRead\x18\x03 \x01(\x08\x12\x16\n\x0e\x41llowedNodeIDs\x18\x04 \x03(\r\x12#\n\x17\x41llowedDataCenterNumIDs\x18\x05 \x03(\rB\x02\x18\x01\x12\x1d\n\x15RequireAllDataCenters\x18\x06 \x01(\x08\x12\x15\n\rLocalNodeOnly\x18\x07 \x01(\x08\x12\x1d\n\x15RequireDifferentNodes\x18\x08 \x01(\x08\x12\"\n\x1a\x46ollowerCountPerDataCenter\x18\t \x01(\x08\x12\x1a\n\x12\x41llowedDataCenters\x18\n \x03(\tB\x18\n\x16ru.yandex.kikimr.proto')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.core.protos.follower_group_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026ru.yandex.kikimr.proto'
  _TFOLLOWERGROUP.fields_by_name['AllowedDataCenterNumIDs']._options = None
  _TFOLLOWERGROUP.fields_by_name['AllowedDataCenterNumIDs']._serialized_options = b'\030\001'
  _TFOLLOWERGROUP._serialized_start=54
  _TFOLLOWERGROUP._serialized_end=358
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/core/protos/tablet_tx.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb.core.protos import tx_proxy_pb2 as ydb_dot_core_dot_protos_dot_tx__proxy__pb2
from ydb.core.protos import scheme_log_pb2 as ydb_dot_core_dot_protos_dot_scheme__log__pb2
from ydb.library.mkql_proto.protos import minikql_pb2 as ydb_dot_library_dot_mkql__proto_dot_protos_dot_minikql__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fydb/core/protos/tablet_tx.proto\x12\x13NKikimrTabletTxBase\x1a\x1eydb/core/protos/tx_proxy.proto\x1a ydb/core/protos/scheme_log.proto\x1a+ydb/library/mkql_proto/protos/minikql.proto\"H\n\x0cTEvLocalMKQL\x12\x38\n\x07Program\x18\x01 \x01(\x0b\x32\'.NKikimrTxUserProxy.TMiniKQLTransaction\"\x88\x02\n\x14TEvLocalMKQLResponse\x12\x0e\n\x06Origin\x18\x01 \x01(\x04\x12\x0e\n\x06Status\x18\x02 \x01(\x05\x12\x14\n\x0c\x45ngineStatus\x18\n \x01(\r\x12\x1c\n\x14\x45ngineResponseStatus\x18\x0b \x01(\r\x12\x15\n\rMiniKQLErrors\x18\x0c \x01(\x0c\x12\x41\n ExecutionEngineEvaluatedResponse\x18\r \x01(\x0b\x32\x17.NKikimrMiniKQL.TResult\x12\x42\n\x0e\x43ompileResults\x18\x0e \x01(\x0b\x32*.NKikimrTxUserProxy.TMiniKQLCompileResults\"\\\n\x10TEvLocalSchemeTx\x12\x38\n\rSchemeChanges\x18\x01 \x01(\x0b\x32!.NTabletFlatScheme.TSchemeChanges\x12\x0e\n\x06\x44ryRun\x18\x02 \x01(\x08\"\x86\x01\n\x18TEvLocalSchemeTxResponse\x12\x0e\n\x06Origin\x18\x01 \x01(\x04\x12\x0e\n\x06Status\x18\x02 \x01(\x05\x12\x13\n\x0b\x45rrorReason\x18\x03 \x01(\t\x12\x35\n\nFullScheme\x18\x04 \x01(\x0b\x32!.NTabletFlatScheme.TSchemeChanges\"\xbe\x01\n\x13TEvLocalReadColumns\x12\x11\n\tTableName\x18\x01 \x01(\t\x12\x0f\n\x07\x43olumns\x18\x02 \x03(\t\x12\x0f\n\x07MaxRows\x18\x03 \x01(\x04\x12\x10\n\x08MaxBytes\x18\x04 \x01(\x04\x12\x0f\n\x07\x46romKey\x18\x05 \x01(\x0c\x12\x18\n\x10\x46romKeyInclusive\x18\x06 \x01(\x08\x12\r\n\x05ToKey\x18\x07 \x01(\x0c\x12\x16\n\x0eToKeyInclusive\x18\x08 \x01(\x08\x12\x0e\n\x06\x46ormat\x18\t \x01(\t\"\xa8\x01\n\x1bTEvLocalReadColumnsResponse\x12\x10\n\x08TabletID\x18\x01 \x01(\x04\x12\x0e\n\x06Status\x18\x02 \x01(\r\x12\x18\n\x10\x45rrorDescription\x18\x03 \x01(\t\x12\x0e\n\x06\x42locks\x18\x04 \x01(\x0c\x12\x0f\n\x07LastKey\x18\x05 \x01(\x0c\x12\x18\n\x10LastKeyInclusive\x18\x06 \x01(\x08\x12\x12\n\nEndOfShard\x18\x07 \x01(\x08\x42\x18\n\x16ru.yandex.kikimr.proto')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.core.protos.tablet_tx_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026ru.yandex.kikimr.proto'
  _TEVLOCALMKQL._serialized_start=167
  _TEVLOCALMKQL._serialized_end=239
  _TEVLOCALMKQLRESPONSE._serialized_start=242
  _TEVLOCALMKQLRESPONSE._serialized_end=506
  _TEVLOCALSCHEMETX._serialized_start=508
  _TEVLOCALSCHEMETX._serialized_end=600
  _TEVLOCALSCHEMETXRESPONSE._serialized_start=603
  _TEVLOCALSCHEMETXRESPONSE._serialized_end=737
  _TEVLOCALREADCOLUMNS._serialized_start=740
  _TEVLOCALREADCOLUMNS._serialized_end=930
  _TEVLOCALREADCOLUMNSRESPONSE._serialized_start=933
  _TEVLOCALREADCOLUMNSRESPONSE._serialized_end=1101
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/public/api/protos/draft/datastreams.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb.public.api.protos import ydb_operation_pb2 as ydb_dot_public_dot_api_dot_protos_dot_ydb__operation__pb2
from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-ydb/public/api/protos/draft/datastreams.proto\x12\x12Ydb.DataStreams.V1\x1a)ydb/public/api/protos/ydb_operation.proto\x1a google/protobuf/descriptor.proto\".\n\x0f\x45nhancedMetrics\x12\x1b\n\x13shard_level_metrics\x18\x01 \x03(\t\"k\n\nChildShard\x12\x38\n\x0ehash_key_range\x18\x01 \x01(\x0b\x32 .Ydb.DataStreams.V1.HashKeyRange\x12\x15\n\rparent_shards\x18\x02 \x03(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\"\xb3\x01\n\x08\x43onsumer\x12\x14\n\x0c\x63onsumer_arn\x18\x01 \x01(\t\x12)\n\x1b\x63onsumer_creation_timestamp\x18\x02 \x01(\x03\x42\x04\xd8\xb0\x1c\x02\x12\x15\n\rconsumer_name\x18\x03 \x01(\t\x12O\n\x0f\x63onsumer_status\x18\x04 \x01(\x0e\x32\x36.Ydb.DataStreams.V1.ConsumerDescription.ConsumerStatus\"B\n\x0cHashKeyRange\x12\x19\n\x11starting_hash_key\x18\x01 \x01(\t\x12\x17\n\x0f\x65nding_hash_key\x18\x02 \x01(\t\"\xac\x01\n\x06Record\x12\x17\n\ttimestamp\x18\x01 \x01(\x03\x42\x04\xd8\xb0\x1c\x02\x12\x12\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x42\x04\xd8\xb0\x1c\x01\x12\x36\n\nencryption\x18\x03 \x01(\x0e\x32\".Ydb.DataStreams.V1.EncryptionType\x12\x15\n\rpartition_key\x18\x04 \x01(\t\x12\x17\n\x0fsequence_number\x18\x05 \x01(\t\x12\r\n\x05\x63odec\x18\x06 \x01(\x05\"\x8c\x05\n\x11StreamDescription\x12;\n\x0f\x65ncryption_type\x18\x01 \x01(\x0e\x32\".Ydb.DataStreams.V1.EncryptionType\x12@\n\x13\x65nhanced_monitoring\x18\x02 \x03(\x0b\x32#.Ydb.DataStreams.V1.EnhancedMetrics\x12\x17\n\x0fhas_more_shards\x18\x03 \x01(\x08\x12\x0e\n\x06key_id\x18\x04 \x01(\t\x12\x1e\n\x16retention_period_hours\x18\x05 \x01(\x05\x12)\n\x06shards\x18\x06 \x03(\x0b\x32\x19.Ydb.DataStreams.V1.Shard\x12\x12\n\nstream_arn\x18\x07 \x01(\t\x12\'\n\x19stream_creation_timestamp\x18\x08 \x01(\x03\x42\x04\xd8\xb0\x1c\x02\x12I\n\rstream_status\x18\t \x01(\x0e\x32\x32.Ydb.DataStreams.V1.StreamDescription.StreamStatus\x12\x13\n\x0bstream_name\x18\n \x01(\t\x12\x1e\n\x16write_quota_kb_per_sec\x18\x0b \x01(\x05\x12\r\n\x05owner\x18\x0c \x01(\t\x12\x18\n\x10storage_limit_mb\x18\r \x01(\x03\x12\x42\n\x13stream_mode_details\x18\x0e \x01(\x0b\x32%.Ydb.DataStreams.V1.StreamModeDetails\"Z\n\x0cStreamStatus\x12\x14\n\x10STATUS_UNDEFINED\x10\x00\x12\x0c\n\x08\x43REATING\x10\x01\x12\x0c\n\x08\x44\x45LETING\x10\x02\x12\n\n\x06\x41\x43TIVE\x10\x03\x12\x0c\n\x08UPDATING\x10\x04\"]\n\x13SequenceNumberRange\x12 \n\x18starting_sequence_number\x18\x01 \x01(\t\x12$\n\x16\x65nding_sequence_number\x18\x02 \x01(\tB\x04\xd8\xb0\x1c\x03\"\xe2\x01\n\x05Shard\x12&\n\x18\x61\x64jacent_parent_shard_id\x18\x01 \x01(\tB\x04\xd8\xb0\x1c\x03\x12\x38\n\x0ehash_key_range\x18\x02 \x01(\x0b\x32 .Ydb.DataStreams.V1.HashKeyRange\x12\x1d\n\x0fparent_shard_id\x18\x03 \x01(\tB\x04\xd8\xb0\x1c\x03\x12\x46\n\x15sequence_number_range\x18\x04 \x01(\x0b\x32\'.Ydb.DataStreams.V1.SequenceNumberRange\x12\x10\n\x08shard_id\x18\x05 \x01(\t\"\xa0\x02\n\x13\x43onsumerDescription\x12\x14\n\x0c\x63onsumer_arn\x18\x01 \x01(\t\x12)\n\x1b\x63onsumer_creation_timestamp\x18\x02 \x01(\x03\x42\x04\xd8\xb0\x1c\x02\x12\x15\n\rconsumer_name\x18\x03 \x01(\t\x12O\n\x0f\x63onsumer_status\x18\x04 \x01(\x0e\x32\x36.Ydb.DataStreams.V1.ConsumerDescription.ConsumerStatus\x12\x12\n\nstream_arn\x18\x05 \x01(\t\"L\n\x0e\x43onsumerStatus\x12\x12\n\x0eSTATUS_UNKNOWN\x10\x00\x12\x0c\n\x08\x43REATING\x10\x01\x12\x0c\n\x08\x44\x45LETING\x10\x02\x12\n\n\x06\x41\x43TIVE\x10\x03\"\x9a\x02\n\x0bShardFilter\x12\x10\n\x08shard_id\x18\x01 \x01(\t\x12\x17\n\ttimestamp\x18\x02 \x01(\x03\x42\x04\xd8\xb0\x1c\x02\x12=\n\x04type\x18\x03 \x01(\x0e\x32/.Ydb.DataStreams.V1.ShardFilter.ShardFilterType\"\xa0\x01\n\x0fShardFilterType\x12\x18\n\x14SHARD_TYPE_UNDEFINED\x10\x00\x12\x12\n\x0e\x41\x46TER_SHARD_ID\x10\x01\x12\x13\n\x0f\x41T_TRIM_HORIZON\x10\x02\x12\x15\n\x11\x46ROM_TRIM_HORIZON\x10\x03\x12\r\n\tAT_LATEST\x10\x04\x12\x10\n\x0c\x41T_TIMESTAMP\x10\x05\x12\x12\n\x0e\x46ROM_TIMESTAMP\x10\x06\"y\n\x10StartingPosition\x12\x17\n\ttimestamp\x18\x01 \x01(\x03\x42\x04\xd8\xb0\x1c\x02\x12\x17\n\x0fsequence_number\x18\x02 \x01(\t\x12\x33\n\x04type\x18\x03 \x01(\x0e\x32%.Ydb.DataStreams.V1.ShardIteratorType\"\x98\x03\n\x18StreamDescriptionSummary\x12\x16\n\x0e\x63onsumer_count\x18\x01 \x01(\x05\x12;\n\x0f\x65ncryption_type\x18\x02 \x01(\x0e\x32\".Ydb.DataStreams.V1.EncryptionType\x12@\n\x13\x65nhanced_monitoring\x18\x03 \x03(\x0b\x32#.Ydb.DataStreams.V1.EnhancedMetrics\x12\x0e\n\x06key_id\x18\x04 \x01(\t\x12\x18\n\x10open_shard_count\x18\x05 \x01(\x05\x12\x1e\n\x16retention_period_hours\x18\x06 \x01(\x05\x12\x12\n\nstream_arn\x18\x07 \x01(\t\x12\'\n\x19stream_creation_timestamp\x18\x08 \x01(\x03\x42\x04\xd8\xb0\x1c\x02\x12\x13\n\x0bstream_name\x18\t \x01(\t\x12I\n\rstream_status\x18\n \x01(\x0e\x32\x32.Ydb.DataStreams.V1.StreamDescription.StreamStatus\"\xbe\x01\n\x15SubscribeToShardEvent\x12\x34\n\x0c\x63hild_shards\x18\x01 \x03(\x0b\x32\x1e.Ydb.DataStreams.V1.ChildShard\x12$\n\x1c\x63ontinuation_sequence_number\x18\x02 \x01(\t\x12\x1c\n\x14millis_behind_latest\x18\x03 \x01(\x05\x12+\n\x07records\x18\x04 \x03(\x0b\x32\x1a.Ydb.DataStreams.V1.Record\"\x90\x08\n\x1bSubscribeToShardEventStream\x12\x64\n\x1ainternal_failure_exception\x18\x01 \x01(\x0b\x32>.Ydb.DataStreams.V1.SubscribeToShardEventStream.ExceptionEventH\x00\x12\x65\n\x1bkms_access_denied_exception\x18\x02 \x01(\x0b\x32>.Ydb.DataStreams.V1.SubscribeToShardEventStream.ExceptionEventH\x00\x12`\n\x16kms_disabled_exception\x18\x03 \x01(\x0b\x32>.Ydb.DataStreams.V1.SubscribeToShardEventStream.ExceptionEventH\x00\x12\x65\n\x1bkms_invalid_state_exception\x18\x04 \x01(\x0b\x32>.Ydb.DataStreams.V1.SubscribeToShardEventStream.ExceptionEventH\x00\x12\x61\n\x17kms_not_found_exception\x18\x05 \x01(\x0b\x32>.Ydb.DataStreams.V1.SubscribeToShardEventStream.ExceptionEventH\x00\x12]\n\x13kms_opt_in_required\x18\x06 \x01(\x0b\x32>.Ydb.DataStreams.V1.SubscribeToShardEventStream.ExceptionEventH\x00\x12\x62\n\x18kms_throttling_exception\x18\x07 \x01(\x0b\x32>.Ydb.DataStreams.V1.SubscribeToShardEventStream.ExceptionEventH\x00\x12\x63\n\x19resource_in_use_exception\x18\x08 \x01(\x0b\x32>.Ydb.DataStreams.V1.SubscribeToShardEventStream.ExceptionEventH\x00\x12\x66\n\x1cresource_not_found_exception\x18\t \x01(\x0b\x32>.Ydb.DataStreams.V1.SubscribeToShardEventStream.ExceptionEventH\x00\x12M\n\x18subscribe_to_shard_event\x18\n \x01(\x0b\x32).Ydb.DataStreams.V1.SubscribeToShardEventH\x00\x1a\x10\n\x0e\x45xceptionEventB\x07\n\x05\x65vent\"!\n\x03Tag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"H\n\x11StreamModeDetails\x12\x33\n\x0bstream_mode\x18\x01 \x01(\x0e\x32\x1e.Ydb.DataStreams.V1.StreamMode\"\xb4\x02\n\x13\x43reateStreamRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x13\n\x0bstream_name\x18\x02 \x01(\t\x12\x13\n\x0bshard_count\x18\x03 \x01(\x05\x12\x1e\n\x16write_quota_kb_per_sec\x18\x04 \x01(\x05\x12 \n\x16retention_period_hours\x18\x05 \x01(\x05H\x00\x12%\n\x1bretention_storage_megabytes\x18\x06 \x01(\x05H\x00\x12\x42\n\x13stream_mode_details\x18\x07 \x01(\x0b\x32%.Ydb.DataStreams.V1.StreamModeDetailsB\x0b\n\tretention\"D\n\x14\x43reateStreamResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x14\n\x12\x43reateStreamResult\"\x88\x01\n\x13\x44\x65leteStreamRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12!\n\x19\x65nforce_consumer_deletion\x18\x02 \x01(\x08\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\"D\n\x14\x44\x65leteStreamResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x14\n\x12\x44\x65leteStreamResult\"\x94\x01\n\x12ListStreamsRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12#\n\x1b\x65xclusive_start_stream_name\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x0f\n\x07recurse\x18\x04 \x01(\x08\"C\n\x13ListStreamsResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"C\n\x11ListStreamsResult\x12\x18\n\x10has_more_streams\x18\x01 \x01(\x08\x12\x14\n\x0cstream_names\x18\x02 \x03(\t\"\x8e\x02\n\x11ListShardsRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12 \n\x18\x65xclusive_start_shard_id\x18\x02 \x01(\t\x12\x13\n\x0bmax_results\x18\x03 \x01(\x05\x12\x12\n\nnext_token\x18\x04 \x01(\t\x12\x35\n\x0cshard_filter\x18\x05 \x01(\x0b\x32\x1f.Ydb.DataStreams.V1.ShardFilter\x12\'\n\x19stream_creation_timestamp\x18\x06 \x01(\x03\x42\x04\xd8\xb0\x1c\x02\x12\x13\n\x0bstream_name\x18\x07 \x01(\t\"B\n\x12ListShardsResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"Q\n\x10ListShardsResult\x12\x12\n\nnext_token\x18\x01 \x01(\t\x12)\n\x06shards\x18\x02 \x03(\x0b\x32\x19.Ydb.DataStreams.V1.Shard\"\x86\x01\n\x14SetWriteQuotaRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x13\n\x0bstream_name\x18\x02 \x01(\t\x12\x1e\n\x16write_quota_kb_per_sec\x18\x03 \x01(\x05\"E\n\x15SetWriteQuotaResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x15\n\x13SetWriteQuotaResult\"\xbb\x02\n\x13UpdateStreamRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x13\n\x0bstream_name\x18\x02 \x01(\t\x12\x1e\n\x16write_quota_kb_per_sec\x18\x03 \x01(\x05\x12\x1a\n\x12target_shard_count\x18\x04 \x01(\x05\x12 \n\x16retention_period_hours\x18\x05 \x01(\x05H\x00\x12%\n\x1bretention_storage_megabytes\x18\x06 \x01(\x05H\x00\x12\x42\n\x13stream_mode_details\x18\x07 \x01(\x0b\x32%.Ydb.DataStreams.V1.StreamModeDetailsB\x0b\n\tretention\"D\n\x14UpdateStreamResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x14\n\x12UpdateStreamResult\"\xbd\x01\n\x1aListStreamConsumersRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x13\n\x0bmax_results\x18\x02 \x01(\x05\x12\x12\n\nnext_token\x18\x03 \x01(\t\x12\x12\n\nstream_arn\x18\x04 \x01(\t\x12\'\n\x19stream_creation_timestamp\x18\x05 \x01(\x03\x42\x04\xd8\xb0\x1c\x02\"K\n\x1bListStreamConsumersResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"`\n\x19ListStreamConsumersResult\x12\x12\n\nnext_token\x18\x01 \x01(\t\x12/\n\tconsumers\x18\x02 \x03(\x0b\x32\x1c.Ydb.DataStreams.V1.Consumer\"\x98\x01\n\x15\x44\x65scribeStreamRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12 \n\x18\x65xclusive_start_shard_id\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x13\n\x0bstream_name\x18\x04 \x01(\t\"F\n\x16\x44\x65scribeStreamResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"Y\n\x14\x44\x65scribeStreamResult\x12\x41\n\x12stream_description\x18\x01 \x01(\x0b\x32%.Ydb.DataStreams.V1.StreamDescription\"\x85\x01\n\x1dRegisterStreamConsumerRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x15\n\rconsumer_name\x18\x02 \x01(\t\x12\x12\n\nstream_arn\x18\x03 \x01(\t\"N\n\x1eRegisterStreamConsumerResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"N\n\x1cRegisterStreamConsumerResult\x12.\n\x08\x63onsumer\x18\x01 \x01(\x0b\x32\x1c.Ydb.DataStreams.V1.Consumer\"\x9d\x01\n\x1f\x44\x65registerStreamConsumerRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x14\n\x0c\x63onsumer_arn\x18\x02 \x01(\t\x12\x15\n\rconsumer_name\x18\x03 \x01(\t\x12\x12\n\nstream_arn\x18\x04 \x01(\t\"P\n DeregisterStreamConsumerResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\" \n\x1e\x44\x65registerStreamConsumerResult\"\x9b\x01\n\x1d\x44\x65scribeStreamConsumerRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x14\n\x0c\x63onsumer_arn\x18\x02 \x01(\t\x12\x15\n\rconsumer_name\x18\x03 \x01(\t\x12\x12\n\nstream_arn\x18\x04 \x01(\t\"N\n\x1e\x44\x65scribeStreamConsumerResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"e\n\x1c\x44\x65scribeStreamConsumerResult\x12\x45\n\x14\x63onsumer_description\x18\x01 \x01(\x0b\x32\'.Ydb.DataStreams.V1.ConsumerDescription\"^\n\x16PutRecordsRequestEntry\x12\x12\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x42\x04\xd8\xb0\x1c\x01\x12\x19\n\x11\x65xplicit_hash_key\x18\x02 \x01(\t\x12\x15\n\rpartition_key\x18\x03 \x01(\t\"y\n\x15PutRecordsResultEntry\x12\x1b\n\rerror_message\x18\x02 \x01(\tB\x04\xd8\xb0\x1c\x03\x12\x18\n\nerror_code\x18\x03 \x01(\tB\x04\xd8\xb0\x1c\x03\x12\x17\n\x0fsequence_number\x18\x04 \x01(\t\x12\x10\n\x08shard_id\x18\x05 \x01(\t\"u\n\x11GetRecordsRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\r\n\x05limit\x18\x02 \x01(\x05\x12\x16\n\x0eshard_iterator\x18\x03 \x01(\t\"B\n\x12GetRecordsResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\xb0\x01\n\x10GetRecordsResult\x12\x34\n\x0c\x63hild_shards\x18\x01 \x03(\x0b\x32\x1e.Ydb.DataStreams.V1.ChildShard\x12\x1c\n\x14millis_behind_latest\x18\x02 \x01(\x03\x12\x1b\n\x13next_shard_iterator\x18\x03 \x01(\t\x12+\n\x07records\x18\x04 \x03(\x0b\x32\x1a.Ydb.DataStreams.V1.Record\"\xce\x01\n\x10PutRecordRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x12\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x42\x04\xd8\xb0\x1c\x01\x12\x19\n\x11\x65xplicit_hash_key\x18\x03 \x01(\t\x12\x15\n\rpartition_key\x18\x04 \x01(\t\x12$\n\x1csequence_number_for_ordering\x18\x05 \x01(\t\x12\x13\n\x0bstream_name\x18\x06 \x01(\t\"A\n\x11PutRecordResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"y\n\x0fPutRecordResult\x12;\n\x0f\x65ncryption_type\x18\x01 \x01(\x0e\x32\".Ydb.DataStreams.V1.EncryptionType\x12\x17\n\x0fsequence_number\x18\x02 \x01(\t\x12\x10\n\x08shard_id\x18\x03 \x01(\t\"\xa0\x01\n\x11PutRecordsRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12;\n\x07records\x18\x02 \x03(\x0b\x32*.Ydb.DataStreams.V1.PutRecordsRequestEntry\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\"B\n\x12PutRecordsResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\xa8\x01\n\x10PutRecordsResult\x12;\n\x0f\x65ncryption_type\x18\x01 \x01(\x0e\x32\".Ydb.DataStreams.V1.EncryptionType\x12\x1b\n\x13\x66\x61iled_record_count\x18\x02 \x01(\x05\x12:\n\x07records\x18\x03 \x03(\x0b\x32).Ydb.DataStreams.V1.PutRecordsResultEntry\"\xfa\x01\n\x17GetShardIteratorRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x10\n\x08shard_id\x18\x02 \x01(\t\x12\x42\n\x13shard_iterator_type\x18\x03 \x01(\x0e\x32%.Ydb.DataStreams.V1.ShardIteratorType\x12 \n\x18starting_sequence_number\x18\x04 \x01(\t\x12\x13\n\x0bstream_name\x18\x05 \x01(\t\x12\x17\n\ttimestamp\x18\x06 \x01(\x03\x42\x04\xd8\xb0\x1c\x02\"H\n\x18GetShardIteratorResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"0\n\x16GetShardIteratorResult\x12\x16\n\x0eshard_iterator\x18\x01 \x01(\t\"\xbd\x01\n\x17SubscribeToShardRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x14\n\x0c\x63onsumer_arn\x18\x02 \x01(\t\x12\x10\n\x08shard_id\x18\x03 \x01(\t\x12?\n\x11starting_position\x18\x04 \x01(\x0b\x32$.Ydb.DataStreams.V1.StartingPosition\"H\n\x18SubscribeToShardResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"_\n\x16SubscribeToShardResult\x12\x45\n\x0c\x65vent_stream\x18\x01 \x01(\x0b\x32/.Ydb.DataStreams.V1.SubscribeToShardEventStream\"v\n\x16\x41\x64\x64TagsToStreamRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x13\n\x0bstream_name\x18\x02 \x01(\t\x12\x0c\n\x04tags\x18\x03 \x03(\t\"G\n\x17\x41\x64\x64TagsToStreamResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x17\n\x15\x41\x64\x64TagsToStreamResult\"\x96\x01\n$DecreaseStreamRetentionPeriodRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x13\n\x0bstream_name\x18\x02 \x01(\t\x12\x1e\n\x16retention_period_hours\x18\x03 \x01(\x05\"U\n%DecreaseStreamRetentionPeriodResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"%\n#DecreaseStreamRetentionPeriodResult\"R\n\x15\x44\x65scribeLimitsRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\"F\n\x16\x44\x65scribeLimitsResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"E\n\x14\x44\x65scribeLimitsResult\x12\x18\n\x10open_shard_count\x18\x01 \x01(\x05\x12\x13\n\x0bshard_limit\x18\x02 \x01(\x05\"n\n\x1c\x44\x65scribeStreamSummaryRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x13\n\x0bstream_name\x18\x02 \x01(\t\"M\n\x1d\x44\x65scribeStreamSummaryResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"o\n\x1b\x44\x65scribeStreamSummaryResult\x12P\n\x1astream_description_summary\x18\x01 \x01(\x0b\x32,.Ydb.DataStreams.V1.StreamDescriptionSummary\"\x8f\x01\n DisableEnhancedMonitoringRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x1b\n\x13shard_level_metrics\x18\x02 \x03(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\"Q\n!DisableEnhancedMonitoringResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x80\x01\n\x1f\x44isableEnhancedMonitoringResult\x12#\n\x1b\x63urrent_shard_level_metrics\x18\x01 \x03(\t\x12#\n\x1b\x64\x65sired_shard_level_metrics\x18\x02 \x03(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\"\x8e\x01\n\x1f\x45nableEnhancedMonitoringRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x1b\n\x13shard_level_metrics\x18\x02 \x03(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\"P\n EnableEnhancedMonitoringResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x7f\n\x1e\x45nableEnhancedMonitoringResult\x12#\n\x1b\x63urrent_shard_level_metrics\x18\x01 \x03(\t\x12#\n\x1b\x64\x65sired_shard_level_metrics\x18\x02 \x03(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\"\x96\x01\n$IncreaseStreamRetentionPeriodRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x1e\n\x16retention_period_hours\x18\x02 \x01(\x05\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\"U\n%IncreaseStreamRetentionPeriodResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"%\n#IncreaseStreamRetentionPeriodResult\"\x9a\x01\n\x18ListTagsForStreamRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x1f\n\x17\x65xclusive_start_tag_key\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x13\n\x0bstream_name\x18\x04 \x01(\t\"I\n\x19ListTagsForStreamResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"W\n\x17ListTagsForStreamResult\x12\x15\n\rhas_more_tags\x18\x01 \x01(\x08\x12%\n\x04tags\x18\x02 \x03(\x0b\x32\x17.Ydb.DataStreams.V1.Tag\"\x9d\x01\n\x12MergeShardsRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x1f\n\x17\x61\x64jacent_shard_to_merge\x18\x02 \x01(\t\x12\x16\n\x0eshard_to_merge\x18\x03 \x01(\t\x12\x13\n\x0bstream_name\x18\x04 \x01(\t\"C\n\x13MergeShardsResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x13\n\x11MergeShardsResult\"\x7f\n\x1bRemoveTagsFromStreamRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x13\n\x0bstream_name\x18\x02 \x01(\t\x12\x10\n\x08tag_keys\x18\x03 \x03(\t\"L\n\x1cRemoveTagsFromStreamResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x1c\n\x1aRemoveTagsFromStreamResult\"\x9a\x01\n\x11SplitShardRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x1d\n\x15new_starting_hash_key\x18\x02 \x01(\t\x12\x16\n\x0eshard_to_split\x18\x03 \x01(\t\x12\x13\n\x0bstream_name\x18\x04 \x01(\t\"B\n\x12SplitShardResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x12\n\x10SplitShardResult\"\xbb\x01\n\x1cStartStreamEncryptionRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12;\n\x0f\x65ncryption_type\x18\x02 \x01(\x0e\x32\".Ydb.DataStreams.V1.EncryptionType\x12\x0e\n\x06key_id\x18\x03 \x01(\t\x12\x13\n\x0bstream_name\x18\x04 \x01(\t\"M\n\x1dStartStreamEncryptionResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x1d\n\x1bStartStreamEncryptionResult\"\xba\x01\n\x1bStopStreamEncryptionRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12;\n\x0f\x65ncryption_type\x18\x02 \x01(\x0e\x32\".Ydb.DataStreams.V1.EncryptionType\x12\x0e\n\x06key_id\x18\x03 \x01(\t\x12\x13\n\x0bstream_name\x18\x04 \x01(\t\"L\n\x1cStopStreamEncryptionResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x1c\n\x1aStopStreamEncryptionResult\"\x92\x02\n\x17UpdateShardCountRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12M\n\x0cscaling_type\x18\x02 \x01(\x0e\x32\x37.Ydb.DataStreams.V1.UpdateShardCountRequest.ScalingType\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\x12\x1a\n\x12target_shard_count\x18\x04 \x01(\x05\"<\n\x0bScalingType\x12\x18\n\x14SCALING_TYPE_UNKNOWN\x10\x00\x12\x13\n\x0fUNIFORM_SCALING\x10\x01\"H\n\x18UpdateShardCountResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"f\n\x16UpdateShardCountResult\x12\x1b\n\x13\x63urrent_shard_count\x18\x01 \x01(\x05\x12\x13\n\x0bstream_name\x18\x02 \x01(\t\x12\x1a\n\x12target_shard_count\x18\x03 \x01(\x05\"\xac\x01\n\x17UpdateStreamModeRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x12\n\nstream_arn\x18\x02 \x01(\t\x12\x42\n\x13stream_mode_details\x18\x03 \x01(\x0b\x32%.Ydb.DataStreams.V1.StreamModeDetails\"H\n\x18UpdateStreamModeResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x18\n\x16UpdateStreamModeResult*\x86\x01\n\x18\x45\x46ieldTransformationType\x12\x12\n\x0eTRANSFORM_NONE\x10\x00\x12\x14\n\x10TRANSFORM_BASE64\x10\x01\x12 \n\x1cTRANSFORM_DOUBLE_S_TO_INT_MS\x10\x02\x12\x1e\n\x1aTRANSFORM_EMPTY_TO_NOTHING\x10\x03*=\n\x0e\x45ncryptionType\x12\x18\n\x14\x45NCRYPTION_UNDEFINED\x10\x00\x12\x08\n\x04NONE\x10\x01\x12\x07\n\x03KMS\x10\x02*G\n\nStreamMode\x12\x19\n\x15STREAM_MODE_UNDEFINED\x10\x00\x12\x0f\n\x0bPROVISIONED\x10\x01\x12\r\n\tON_DEMAND\x10\x02*\x94\x01\n\x11ShardIteratorType\x12\x1c\n\x18SHARD_ITERATOR_UNDEFINED\x10\x00\x12\x16\n\x12\x41T_SEQUENCE_NUMBER\x10\x01\x12\x19\n\x15\x41\x46TER_SEQUENCE_NUMBER\x10\x02\x12\x10\n\x0c\x41T_TIMESTAMP\x10\x03\x12\x10\n\x0cTRIM_HORIZON\x10\x04\x12\n\n\x06LATEST\x10\x05:g\n\x10\x46ieldTransformer\x12\x1d.google.protobuf.FieldOptions\x18\x8b\xc6\x03 \x01(\x0e\x32,.Ydb.DataStreams.V1.EFieldTransformationTypeB\"\n\x1d\x63om.yandex.ydb.datastreams.v1\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.public.api.protos.draft.datastreams_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(FieldTransformer)

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\035com.yandex.ydb.datastreams.v1\370\001\001'
  _CONSUMER.fields_by_name['consumer_creation_timestamp']._options = None
  _CONSUMER.fields_by_name['consumer_creation_timestamp']._serialized_options = b'\330\260\034\002'
  _RECORD.fields_by_name['timestamp']._options = None
  _RECORD.fields_by_name['timestamp']._serialized_options = b'\330\260\034\002'
  _RECORD.fields_by_name['data']._options = None
  _RECORD.fields_by_name['data']._serialized_options = b'\330\260\034\001'
  _STREAMDESCRIPTION.fields_by_name['stream_creation_timestamp']._options = None
  _STREAMDESCRIPTION.fields_by_name['stream_creation_timestamp']._serialized_options = b'\330\260\034\002'
  _SEQUENCENUMBERRANGE.fields_by_name['ending_sequence_number']._options = None
  _SEQUENCENUMBERRANGE.fields_by_name['ending_sequence_number']._serialized_options = b'\330\260\034\003'
  _SHARD.fields_by_name['adjacent_parent_shard_id']._options = None
  _SHARD.fields_by_name['adjacent_parent_shard_id']._serialized_options = b'\330\260\034\003'
  _SHARD.fields_by_name['parent_shard_id']._options = None
  _SHARD.fields_by_name['parent_shard_id']._serialized_options = b'\330\260\034\003'
  _CONSUMERDESCRIPTION.fields_by_name['consumer_creation_timestamp']._options = None
  _CONSUMERDESCRIPTION.fields_by_name['consumer_creation_timestamp']._serialized_options = b'\330\260\034\002'
  _SHARDFILTER.fields_by_name['timestamp']._options = None
  _SHARDFILTER.fields_by_name['timestamp']._serialized_options = b'\330\260\034\002'
  _STARTINGPOSITION.fields_by_name['timestamp']._options = None
  _STARTINGPOSITION.fields_by_name['timestamp']._serialized_options = b'\330\260\034\002'
  _STREAMDESCRIPTIONSUMMARY.fields_by_name['stream_creation_timestamp']._options = None
  _STREAMDESCRIPTIONSUMMARY.fields_by_name['stream_creation_timestamp']._serialized_options = b'\330\260\034\002'
  _LISTSHARDSREQUEST.fields_by_name['stream_creation_timestamp']._options = None
  _LISTSHARDSREQUEST.fields_by_name['stream_creation_timestamp']._serialized_options = b'\330\260\034\002'
  _LISTSTREAMCONSUMERSREQUEST.fields_by_name['stream_creation_timestamp']._options = None
  _LISTSTREAMCONSUMERSREQUEST.fields_by_name['stream_creation_timestamp']._serialized_options = b'\330\260\034\002'
  _PUTRECORDSREQUESTENTRY.fields_by_name['data']._options = None
  _PUTRECORDSREQUESTENTRY.fields_by_name['data']._serialized_options = b'\330\260\034\001'
  _PUTRECORDSRESULTENTRY.fields_by_name['error_message']._options = None
  _PUTRECORDSRESULTENTRY.fields_by_name['error_message']._serialized_options = b'\330\260\034\003'
  _PUTRECORDSRESULTENTRY.fields_by_name['error_code']._options = None
  _PUTRECORDSRESULTENTRY.fields_by_name['error_code']._serialized_options = b'\330\260\034\003'
  _PUTRECORDREQUEST.fields_by_name['data']._options = None
  _PUTRECORDREQUEST.fields_by_name['data']._serialized_options = b'\330\260\034\001'
  _GETSHARDITERATORREQUEST.fields_by_name['timestamp']._options = None
  _GETSHARDITERATORREQUEST.fields_by_name['timestamp']._serialized_options = b'\330\260\034\002'
  _EFIELDTRANSFORMATIONTYPE._serialized_start=14278
  _EFIELDTRANSFORMATIONTYPE._serialized_end=14412
  _ENCRYPTIONTYPE._serialized_start=14414
  _ENCRYPTIONTYPE._serialized_end=14475
  _STREAMMODE._serialized_start=14477
  _STREAMMODE._serialized_end=14548
  _SHARDITERATORTYPE._serialized_start=14551
  _SHARDITERATORTYPE._serialized_end=14699
  _ENHANCEDMETRICS._serialized_start=146
  _ENHANCEDMETRICS._serialized_end=192
  _CHILDSHARD._serialized_start=194
  _CHILDSHARD._serialized_end=301
  _CONSUMER._serialized_start=304
  _CONSUMER._serialized_end=483
  _HASHKEYRANGE._serialized_start=485
  _HASHKEYRANGE._serialized_end=551
  _RECORD._serialized_start=554
  _RECORD._serialized_end=726
  _STREAMDESCRIPTION._serialized_start=729
  _STREAMDESCRIPTION._serialized_end=1381
  _STREAMDESCRIPTION_STREAMSTATUS._serialized_start=1291
  _STREAMDESCRIPTION_STREAMSTATUS._serialized_end=1381
  _SEQUENCENUMBERRANGE._serialized_start=1383
  _SEQUENCENUMBERRANGE._serialized_end=1476
  _SHARD._serialized_start=1479
  _SHARD._serialized_end=1705
  _CONSUMERDESCRIPTION._serialized_start=1708
  _CONSUMERDESCRIPTION._serialized_end=1996
  _CONSUMERDESCRIPTION_CONSUMERSTATUS._serialized_start=1920
  _CONSUMERDESCRIPTION_CONSUMERSTATUS._serialized_end=1996
  _SHARDFILTER._serialized_start=1999
  _SHARDFILTER._serialized_end=2281
  _SHARDFILTER_SHARDFILTERTYPE._serialized_start=2121
  _SHARDFILTER_SHARDFILTERTYPE._serialized_end=2281
  _STARTINGPOSITION._serialized_start=2283
  _STARTINGPOSITION._serialized_end=2404
  _STREAMDESCRIPTIONSUMMARY._serialized_start=2407
  _STREAMDESCRIPTIONSUMMARY._serialized_end=2815
  _SUBSCRIBETOSHARDEVENT._serialized_start=2818
  _SUBSCRIBETOSHARDEVENT._serialized_end=3008
  _SUBSCRIBETOSHARDEVENTSTREAM._serialized_start=3011
  _SUBSCRIBETOSHARDEVENTSTREAM._serialized_end=4051
  _SUBSCRIBETOSHARDEVENTSTREAM_EXCEPTIONEVENT._serialized_start=4026
  _SUBSCRIBETOSHARDEVENTSTREAM_EXCEPTIONEVENT._serialized_end=4042
  _TAG._serialized_start=4053
  _TAG._serialized_end=4086
  _STREAMMODEDETAILS._serialized_start=4088
  _STREAMMODEDETAILS._serialized_end=4160
  _CREATESTREAMREQUEST._serialized_start=4163
  _CREATESTREAMREQUEST._serialized_end=4471
  _CREATESTREAMRESPONSE._serialized_start=4473
  _CREATESTREAMRESPONSE._serialized_end=4541
  _CREATESTREAMRESULT._serialized_start=4543
  _CREATESTREAMRESULT._serialized_end=4563
  _DELETESTREAMREQUEST._serialized_start=4566
  _DELETESTREAMREQUEST._serialized_end=4702
  _DELETESTREAMRESPONSE._serialized_start=4704
  _DELETESTREAMRESPONSE._serialized_end=4772
  _DELETESTREAMRESULT._serialized_start=4774
  _DELETESTREAMRESULT._serialized_end=4794
  _LISTSTREAMSREQUEST._serialized_start=4797
  _LISTSTREAMSREQUEST._serialized_end=4945
  _LISTSTREAMSRESPONSE._serialized_start=4947
  _LISTSTREAMSRESPONSE._serialized_end=5014
  _LISTSTREAMSRESULT._serialized_start=5016
  _LISTSTREAMSRESULT._serialized_end=5083
  _LISTSHARDSREQUEST._serialized_start=5086
  _LISTSHARDSREQUEST._serialized_end=5356
  _LISTSHARDSRESPONSE._serialized_start=5358
  _LISTSHARDSRESPONSE._serialized_end=5424
  _LISTSHARDSRESULT._serialized_start=5426
  _LISTSHARDSRESULT._serialized_end=5507
  _SETWRITEQUOTAREQUEST._serialized_start=5510
  _SETWRITEQUOTAREQUEST._serialized_end=5644
  _SETWRITEQUOTARESPONSE._serialized_start=5646
  _SETWRITEQUOTARESPONSE._serialized_end=5715
  _SETWRITEQUOTARESULT._serialized_start=5717
  _SETWRITEQUOTARESULT._serialized_end=5738
  _UPDATESTREAMREQUEST._serialized_start=5741
  _UPDATESTREAMREQUEST._serialized_end=6056
  _UPDATESTREAMRESPONSE._serialized_start=6058
  _UPDATESTREAMRESPONSE._serialized_end=6126
  _UPDATESTREAMRESULT._serialized_start=6128
  _UPDATESTREAMRESULT._serialized_end=6148
  _LISTSTREAMCONSUMERSREQUEST._serialized_start=6151
  _LISTSTREAMCONSUMERSREQUEST._serialized_end=6340
  _LISTSTREAMCONSUMERSRESPONSE._serialized_start=6342
  _LISTSTREAMCONSUMERSRESPONSE._serialized_end=6417
  _LISTSTREAMCONSUMERSRESULT._serialized_start=6419
  _LISTSTREAMCONSUMERSRESULT._serialized_end=6515
  _DESCRIBESTREAMREQUEST._serialized_start=6518
  _DESCRIBESTREAMREQUEST._serialized_end=6670
  _DESCRIBESTREAMRESPONSE._serialized_start=6672
  _DESCRIBESTREAMRESPONSE._serialized_end=6742
  _DESCRIBESTREAMRESULT._serialized_start=6744
  _DESCRIBESTREAMRESULT._serialized_end=6833
  _REGISTERSTREAMCONSUMERREQUEST._serialized_start=6836
  _REGISTERSTREAMCONSUMERREQUEST._serialized_end=6969
  _REGISTERSTREAMCONSUMERRESPONSE._serialized_start=6971
  _REGISTERSTREAMCONSUMERRESPONSE._serialized_end=7049
  _REGISTERSTREAMCONSUMERRESULT._serialized_start=7051
  _REGISTERSTREAMCONSUMERRESULT._serialized_end=7129
  _DEREGISTERSTREAMCONSUMERREQUEST._serialized_start=7132
  _DEREGISTERSTREAMCONSUMERREQUEST._serialized_end=7289
  _DEREGISTERSTREAMCONSUMERRESPONSE._serialized_start=7291
  _DEREGISTERSTREAMCONSUMERRESPONSE._serialized_end=7371
  _DEREGISTERSTREAMCONSUMERRESULT._serialized_start=7373
  _DEREGISTERSTREAMCONSUMERRESULT._serialized_end=7405
  _DESCRIBESTREAMCONSUMERREQUEST._serialized_start=7408
  _DESCRIBESTREAMCONSUMERREQUEST._serialized_end=7563
  _DESCRIBESTREAMCONSUMERRESPONSE._serialized_start=7565
  _DESCRIBESTREAMCONSUMERRESPONSE._serialized_end=7643
  _DESCRIBESTREAMCONSUMERRESULT._serialized_start=7645
  _DESCRIBESTREAMCONSUMERRESULT._serialized_end=7746
  _PUTRECORDSREQUESTENTRY._serialized_start=7748
  _PUTRECORDSREQUESTENTRY._serialized_end=7842
  _PUTRECORDSRESULTENTRY._serialized_start=7844
  _PUTRECORDSRESULTENTRY._serialized_end=7965
  _GETRECORDSREQUEST._serialized_start=7967
  _GETRECORDSREQUEST._serialized_end=8084
  _GETRECORDSRESPONSE._serialized_start=8086
  _GETRECORDSRESPONSE._serialized_end=8152
  _GETRECORDSRESULT._serialized_start=8155
  _GETRECORDSRESULT._serialized_end=8331
  _PUTRECORDREQUEST._serialized_start=8334
  _PUTRECORDREQUEST._serialized_end=8540
  _PUTRECORDRESPONSE._serialized_start=8542
  _PUTRECORDRESPONSE._serialized_end=8607
  _PUTRECORDRESULT._serialized_start=8609
  _PUTRECORDRESULT._serialized_end=8730
  _PUTRECORDSREQUEST._serialized_start=8733
  _PUTRECORDSREQUEST._serialized_end=8893
  _PUTRECORDSRESPONSE._serialized_start=8895
  _PUTRECORDSRESPONSE._serialized_end=8961
  _PUTRECORDSRESULT._serialized_start=8964
  _PUTRECORDSRESULT._serialized_end=9132
  _GETSHARDITERATORREQUEST._serialized_start=9135
  _GETSHARDITERATORREQUEST._serialized_end=9385
  _GETSHARDITERATORRESPONSE._serialized_start=9387
  _GETSHARDITERATORRESPONSE._serialized_end=9459
  _GETSHARDITERATORRESULT._serialized_start=9461
  _GETSHARDITERATORRESULT._serialized_end=9509
  _SUBSCRIBETOSHARDREQUEST._serialized_start=9512
  _SUBSCRIBETOSHARDREQUEST._serialized_end=9701
  _SUBSCRIBETOSHARDRESPONSE._serialized_start=9703
  _SUBSCRIBETOSHARDRESPONSE._serialized_end=9775
  _SUBSCRIBETOSHARDRESULT._serialized_start=9777
  _SUBSCRIBETOSHARDRESULT._serialized_end=9872
  _ADDTAGSTOSTREAMREQUEST._serialized_start=9874
  _ADDTAGSTOSTREAMREQUEST._serialized_end=9992
  _ADDTAGSTOSTREAMRESPONSE._serialized_start=9994
  _ADDTAGSTOSTREAMRESPONSE._serialized_end=10065
  _ADDTAGSTOSTREAMRESULT._serialized_start=10067
  _ADDTAGSTOSTREAMRESULT._serialized_end=10090
  _DECREASESTREAMRETENTIONPERIODREQUEST._serialized_start=10093
  _DECREASESTREAMRETENTIONPERIODREQUEST._serialized_end=10243
  _DECREASESTREAMRETENTIONPERIODRESPONSE._serialized_start=10245
  _DECREASESTREAMRETENTIONPERIODRESPONSE._serialized_end=10330
  _DECREASESTREAMRETENTIONPERIODRESULT._serialized_start=10332
  _DECREASESTREAMRETENTIONPERIODRESULT._serialized_end=10369
  _DESCRIBELIMITSREQUEST._serialized_start=10371
  _DESCRIBELIMITSREQUEST._serialized_end=10453
  _DESCRIBELIMITSRESPONSE._serialized_start=10455
  _DESCRIBELIMITSRESPONSE._serialized_end=10525
  _DESCRIBELIMITSRESULT._serialized_start=10527
  _DESCRIBELIMITSRESULT._serialized_end=10596
  _DESCRIBESTREAMSUMMARYREQUEST._serialized_start=10598
  _DESCRIBESTREAMSUMMARYREQUEST._serialized_end=10708
  _DESCRIBESTREAMSUMMARYRESPONSE._serialized_start=10710
  _DESCRIBESTREAMSUMMARYRESPONSE._serialized_end=10787
  _DESCRIBESTREAMSUMMARYRESULT._serialized_start=10789
  _DESCRIBESTREAMSUMMARYRESULT._serialized_end=10900
  _DISABLEENHANCEDMONITORINGREQUEST._serialized_start=10903
  _DISABLEENHANCEDMONITORINGREQUEST._serialized_end=11046
  _DISABLEENHANCEDMONITORINGRESPONSE._serialized_start=11048
  _DISABLEENHANCEDMONITORINGRESPONSE._serialized_end=11129
  _DISABLEENHANCEDMONITORINGRESULT._serialized_start=11132
  _DISABLEENHANCEDMONITORINGRESULT._serialized_end=11260
  _ENABLEENHANCEDMONITORINGREQUEST._serialized_start=11263
  _ENABLEENHANCEDMONITORINGREQUEST._serialized_end=11405
  _ENABLEENHANCEDMONITORINGRESPONSE._serialized_start=11407
  _ENABLEENHANCEDMONITORINGRESPONSE._serialized_end=11487
  _ENABLEENHANCEDMONITORINGRESULT._serialized_start=11489
  _ENABLEENHANCEDMONITORINGRESULT._serialized_end=11616
  _INCREASESTREAMRETENTIONPERIODREQUEST._serialized_start=11619
  _INCREASESTREAMRETENTIONPERIODREQUEST._serialized_end=11769
  _INCREASESTREAMRETENTIONPERIODRESPONSE._serialized_start=11771
  _INCREASESTREAMRETENTIONPERIODRESPONSE._serialized_end=11856
  _INCREASESTREAMRETENTIONPERIODRESULT._serialized_start=11858
  _INCREASESTREAMRETENTIONPERIODRESULT._serialized_end=11895
  _LISTTAGSFORSTREAMREQUEST._serialized_start=11898
  _LISTTAGSFORSTREAMREQUEST._serialized_end=12052
  _LISTTAGSFORSTREAMRESPONSE._serialized_start=12054
  _LISTTAGSFORSTREAMRESPONSE._serialized_end=12127
  _LISTTAGSFORSTREAMRESULT._serialized_start=12129
  _LISTTAGSFORSTREAMRESULT._serialized_end=12216
  _MERGESHARDSREQUEST._serialized_start=12219
  _MERGESHARDSREQUEST._serialized_end=12376
  _MERGESHARDSRESPONSE._serialized_start=12378
  _MERGESHARDSRESPONSE._serialized_end=12445
  _MERGESHARDSRESULT._serialized_start=12447
  _MERGESHARDSRESULT._serialized_end=12466
  _REMOVETAGSFROMSTREAMREQUEST._serialized_start=12468
  _REMOVETAGSFROMSTREAMREQUEST._serialized_end=12595
  _REMOVETAGSFROMSTREAMRESPONSE._serialized_start=12597
  _REMOVETAGSFROMSTREAMRESPONSE._serialized_end=12673
  _REMOVETAGSFROMSTREAMRESULT._serialized_start=12675
  _REMOVETAGSFROMSTREAMRESULT._serialized_end=12703
  _SPLITSHARDREQUEST._serialized_start=12706
  _SPLITSHARDREQUEST._serialized_end=12860
  _SPLITSHARDRESPONSE._serialized_start=12862
  _SPLITSHARDRESPONSE._serialized_end=12928
  _SPLITSHARDRESULT._serialized_start=12930
  _SPLITSHARDRESULT._serialized_end=12948
  _STARTSTREAMENCRYPTIONREQUEST._serialized_start=12951
  _STARTSTREAMENCRYPTIONREQUEST._serialized_end=13138
  _STARTSTREAMENCRYPTIONRESPONSE._serialized_start=13140
  _STARTSTREAMENCRYPTIONRESPONSE._serialized_end=13217
  _STARTSTREAMENCRYPTIONRESULT._serialized_start=13219
  _STARTSTREAMENCRYPTIONRESULT._serialized_end=13248
  _STOPSTREAMENCRYPTIONREQUEST._serialized_start=13251
  _STOPSTREAMENCRYPTIONREQUEST._serialized_end=13437
  _STOPSTREAMENCRYPTIONRESPONSE._serialized_start=13439
  _STOPSTREAMENCRYPTIONRESPONSE._serialized_end=13515
  _STOPSTREAMENCRYPTIONRESULT._serialized_start=13517
  _STOPSTREAMENCRYPTIONRESULT._serialized_end=13545
  _UPDATESHARDCOUNTREQUEST._serialized_start=13548
  _UPDATESHARDCOUNTREQUEST._serialized_end=13822
  _UPDATESHARDCOUNTREQUEST_SCALINGTYPE._serialized_start=13762
  _UPDATESHARDCOUNTREQUEST_SCALINGTYPE._serialized_end=13822
  _UPDATESHARDCOUNTRESPONSE._serialized_start=13824
  _UPDATESHARDCOUNTRESPONSE._serialized_end=13896
  _UPDATESHARDCOUNTRESULT._serialized_start=13898
  _UPDATESHARDCOUNTRESULT._serialized_end=14000
  _UPDATESTREAMMODEREQUEST._serialized_start=14003
  _UPDATESTREAMMODEREQUEST._serialized_end=14175
  _UPDATESTREAMMODERESPONSE._serialized_start=14177
  _UPDATESTREAMMODERESPONSE._serialized_end=14249
  _UPDATESTREAMMODERESULT._serialized_start=14251
  _UPDATESTREAMMODERESULT._serialized_end=14275
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/core/protos/channel_purpose.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%ydb/core/protos/channel_purpose.proto\x12\x16NKikimrStorageSettings\"\x9e\x01\n\x0fTChannelPurpose\x12\x41\n\x07Purpose\x18\x01 \x01(\x0e\x32\x30.NKikimrStorageSettings.TChannelPurpose.EPurpose\x12\x0f\n\x07\x43hannel\x18\x02 \x01(\r\"7\n\x08\x45Purpose\x12\n\n\x06SysLog\x10\x01\x12\x07\n\x03Log\x10\x02\x12\x08\n\x04\x44\x61ta\x10\x03\x12\x0c\n\x08\x45xternal\x10\x04\"\\\n\x0cTStorageRoom\x12\x0e\n\x06RoomId\x18\x01 \x01(\r\x12<\n\x0b\x45xplanation\x18\x02 \x03(\x0b\x32\'.NKikimrStorageSettings.TChannelPurposeB\x18\n\x16ru.yandex.kikimr.proto')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.core.protos.channel_purpose_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026ru.yandex.kikimr.proto'
  _TCHANNELPURPOSE._serialized_start=66
  _TCHANNELPURPOSE._serialized_end=224
  _TCHANNELPURPOSE_EPURPOSE._serialized_start=169
  _TCHANNELPURPOSE_EPURPOSE._serialized_end=224
  _TSTORAGEROOM._serialized_start=226
  _TSTORAGEROOM._serialized_end=318
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: library/cpp/actors/protos/actors.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&library/cpp/actors/protos/actors.proto\x12\x0cNActorsProto\"(\n\x08TActorId\x12\r\n\x05RawX1\x18\x01 \x02(\x06\x12\r\n\x05RawX2\x18\x02 \x02(\x06\"\x18\n\x08TTraceId\x12\x0c\n\x04\x44\x61ta\x18\x01 \x01(\x0c\"W\n\x12TCallbackException\x12\'\n\x07\x41\x63torId\x18\x01 \x02(\x0b\x32\x16.NActorsProto.TActorId\x12\x18\n\x10\x45xceptionMessage\x18\x02 \x02(\t\"\xee\x02\n\x0fTRemoteHttpInfo\x12\x0e\n\x06Method\x18\x01 \x01(\r\x12\x0c\n\x04Path\x18\x02 \x01(\t\x12>\n\x0bQueryParams\x18\x03 \x03(\x0b\x32).NActorsProto.TRemoteHttpInfo.TQueryParam\x12=\n\nPostParams\x18\x04 \x03(\x0b\x32).NActorsProto.TRemoteHttpInfo.TQueryParam\x12\x13\n\x0bPostContent\x18\x08 \x01(\x0c\x12\x36\n\x07Headers\x18\t \x03(\x0b\x32%.NActorsProto.TRemoteHttpInfo.THeader\x12\x12\n\nRemoteAddr\x18\x07 \x01(\t\x1a)\n\x0bTQueryParam\x12\x0b\n\x03Key\x18\x01 \x01(\t\x12\r\n\x05Value\x18\x02 \x01(\t\x1a&\n\x07THeader\x12\x0c\n\x04Name\x18\x01 \x01(\t\x12\r\n\x05Value\x18\x02 \x01(\tJ\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07\x42*\n\x16ru.yandex.kikimr.protoB\x10NActorsBaseProto')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'library.cpp.actors.protos.actors_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026ru.yandex.kikimr.protoB\020NActorsBaseProto'
  _TACTORID._serialized_start=56
  _TACTORID._serialized_end=96
  _TTRACEID._serialized_start=98
  _TTRACEID._serialized_end=122
  _TCALLBACKEXCEPTION._serialized_start=124
  _TCALLBACKEXCEPTION._serialized_end=211
  _TREMOTEHTTPINFO._serialized_start=214
  _TREMOTEHTTPINFO._serialized_end=580
  _TREMOTEHTTPINFO_TQUERYPARAM._serialized_start=487
  _TREMOTEHTTPINFO_TQUERYPARAM._serialized_end=528
  _TREMOTEHTTPINFO_THEADER._serialized_start=530
  _TREMOTEHTTPINFO_THEADER._serialized_end=568
# @@protoc_insertion_point(module_scope)



from ydb.apps.dstool.lib.bs_layout import BlobStorageLayout
import ydb.apps.dstool.lib.common as common
import uuid
from threading import Thread, Lock
import struct
from argparse import FileType

description = 'Take snapshot of groups metadata'
lock = Lock()
output_file = None


def fetch_blobs_from_vdisk(group_id, index, host, pdisk_id, vslot_id):
    session_id = uuid.uuid4()
    params = dict(pdiskId=pdisk_id, vdiskSlotId=vslot_id, sessionId=session_id)
    while True:
        data = common.fetch('vdisk_stream', params, explicit_host=host, fmt='raw')
        if not data or data == b'ERROR':
            break
        with lock:
            output_file.write(struct.pack('@III', group_id, index, len(data)))
            output_file.write(data)


def add_options(p):
    common.add_group_ids_option(p, required=True)
    p.add_argument('--output', type=FileType('wb'), required=True, help='Path to output binary file')
    common.add_basic_format_options(p)


def do(args):
    def get_endpoints():
        layout = BlobStorageLayout()
        layout.fetch_node_mon_endpoints({vslot_id.pdisk_id.node_id for vslot_id in layout.vslots})

        for group in layout.groups.values():
            if group.base.GroupId in args.group_ids:
                for index, vslot in enumerate(group.vslots_of_group):
                    id_ = vslot.base.VSlotId
                    yield group.base.GroupId, index, vslot.pdisk.node.node_mon_endpoint, id_.PDiskId, id_.VSlotId

    global output_file
    output_file = args.output

    threads = []
    for p in get_endpoints():
        thread = Thread(target=fetch_blobs_from_vdisk, args=p, daemon=True)
        threads.append(thread)
        thread.start()
    for thread in threads:
        thread.join()



from google.protobuf import text_format
import ydb.apps.dstool.lib.common as common
import ydb.core.protos.blobstorage_config_pb2 as kikimr_bsconfig
import sys

description = 'Add disk to storage box by serial number'


def add_options(p):
    p.add_argument('--serial', type=str, required=True, help='Disk serial number')
    p.add_argument('--box', type=int, required=True, help='Box of PDisk')
    p.add_argument('--kind', type=int, help='Kind of PDisk')
    types = kikimr_bsconfig.EPDiskType.keys()
    p.add_argument('--pdisk-type', type=str, choices=types, default='UNKNOWN_TYPE', help='Type of PDisk')
    p.add_argument('--pdisk-config', type=str, metavar='TEXT_PROTOBUF', help='Proto config for PDisk')
    common.add_basic_format_options(p)


def create_request(args):
    request = common.kikimr_bsconfig.TConfigRequest(Rollback=args.dry_run)
    cmd = request.Command.add().AddDriveSerial
    cmd.Serial = args.serial
    cmd.BoxId = args.box
    if args.kind:
        cmd.Kind = args.kind
    if args.pdisk_type:
        cmd.PDiskType = kikimr_bsconfig.EPDiskType.Value(args.pdisk_type)
    if args.pdisk_config:
        text_format.Parse(args.pdisk_config, cmd.PDiskConfig)
    return request


def perform_request(request):
    return common.invoke_bsc_request(request)


def is_successful_response(response):
    return common.is_successful_bsc_response(response)


def do(args):
    request = create_request(args)
    response = perform_request(request)
    common.print_request_result(args, request, response)
    if not is_successful_response(response):
        fail_reasons = set([status.FailReason for status in response.Status])
        if len(fail_reasons) == 1:
            sys.exit(100 + list(fail_reasons)[0])
        else:
            sys.exit(1)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb/core/protos/counters_mediator.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb.core.protos import counters_pb2 as ydb_dot_core_dot_protos_dot_counters__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'ydb/core/protos/counters_mediator.proto\x12\x13NKikimr.NTxMediator\x1a\x1eydb/core/protos/counters.proto*R\n\x08\x45TxTypes\x12\x1d\n\x0bTXTYPE_INIT\x10\x00\x1a\x0c\x8a\xd6\x1b\x08\n\x06TxInit\x12\'\n\x10TXTYPE_CONFIGURE\x10\x01\x1a\x11\x8a\xd6\x1b\r\n\x0bTxConfigureB$\n\x16ru.yandex.kikimr.proto\x9a\xd6\x1b\x08Mediator')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb.core.protos.counters_mediator_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026ru.yandex.kikimr.proto\232\326\033\010Mediator'
  _ETXTYPES.values_by_name["TXTYPE_INIT"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_INIT"]._serialized_options = b'\212\326\033\010\n\006TxInit'
  _ETXTYPES.values_by_name["TXTYPE_CONFIGURE"]._options = None
  _ETXTYPES.values_by_name["TXTYPE_CONFIGURE"]._serialized_options = b'\212\326\033\r\n\013TxConfigure'
  _ETXTYPES._serialized_start=96
  _ETXTYPES._serialized_end=178
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3

from ydb.apps.dstool.lib.arg_parser import ArgumentParser
import ydb.apps.dstool.lib.common as common
import ydb.apps.dstool.lib.commands as commands


def main():
    parser = ArgumentParser(description='YDB Distributed Storage Administration Tool')

    # common options
    common.add_host_access_options(parser)
    parser.add_argument('--dry-run', '-n', action='store_true', help='Run command without side effects')

    subparsers = parser.add_subparsers(help='Subcommands', dest='global_command', required=True)
    command_map = commands.make_command_map_by_structure(subparsers)
    args = parser.parse_args()
    common.apply_args(args)
    commands.run_command(command_map, args)


if __name__ == '__main__':
    main()
